////////////////////////////////////////////////////////////////////////////
//
// stack.h - templatized stack class
//
////////////////////////////////////////////////////////////////////////////
#ifndef _STACK_H_
#define _STACK_H_

#include <stream.h>
#include "SimpleObject.h"

template <class T> class Stack : public SimpleObject {
 public:
  void Expand(); // make stack bigger

  // return number of elements in stack
  int NumElements() const { return (_top - _stack); }

  // is the stack empty?
  Boolean IsEmpty() const { return MakeBoolean(_top == _stack); }

  // is the stack full?
  Boolean IsFull() const { return MakeBoolean(_top == _stack + _size - 1); }

  // check to see if p is a legal pointer to an element of the stack
  Boolean LegalStackPtr(T* p) const { return MakeBoolean(p <= _top && p > _stack); }

  // empty the stack
  void Clear() { _top = _stack; }

  // return pointer to topmost element in stack
  T* TopPtr() const { return _top; }

  // top stack element
  T Top() const { return *_top; }

  // pop top stack element
  T Pop() { assert(_top > _stack); return *_top--; }

  // pop top n elements
  void PopN(int n) { assert(_top - n >= _stack); _top -= n; }

  // push element onto stack
  void Push(T new_top) { if(IsFull()) Expand(); *++_top = new_top; }

  // new stack (d is initial size)
  Stack(int d = 1) : _size(++d) { assert(_top = _stack = new T[_size]); }

  // destroy a stack
  ~Stack() { delete[] _stack; }

  // print stack
  ostream& Print(ostream&, int=0) const;
 private:
  int _size;   // size of stack
  T* _top;     // top ptr
  T* _stack;   // ptr to bottom of stack
};

// expand stack
template <class T>
inline void Stack<T>::Expand()
{
  int top_index = _top - _stack;
  T* new_stack = new T[(_size *= 2)];

  // first element is always empty
  for(unsigned int i = 1; i <= top_index; i++)
    new_stack[i] = _stack[i];
  delete[] _stack;

  _stack = new_stack;
  _top = _stack + top_index;
}

// print stack
template <class T>
inline ostream& Stack<T>::Print(ostream& os, int) const
{
  os << "Stack<T>: ";
  for(T* p = _top; p > _stack; p--)
    os << *p << " ";
  return os;
}

#endif
