#include <stream.h>
#include <stdio.h>
#include "shash.h"
#include "String.h"

// primes used for hashing function
#define Prime1			37
#define Prime2			1048583

// maximum prefix of string used to compute hash value
static unsigned int hash_len = 20;

// compute hash value for s
unsigned int HashString(const char*& s)
{
  register unsigned int h = 0;
  register const char *k = s;
  register int num = 0;

  while (*k && num++ < hash_len)
    h = h*Prime1 ^ (*k++ - ' ');
  return h;
}

// compare two strings
Boolean StringCompare(const char*& a, const char*& b)
{
  return MakeBoolean(!strcmp(a, b));
}

// global string table
static StringTable<char*>* Strings = 0;

// enter string into the string table and
// make sure the table is initialized
const char* rstrlit(const char* s)
{
  if(!Strings)
    Strings = new StringTable<char*>;
  return strlit((char*)s);
}

// enter string into the string table and/or lookup existing string
const char* strlit(const char* s)
{
  if(s) {
    char* hs = (char*)Strings->Lookup((char*)s);
    if(!hs) {
      hs = new char[strlen(s)+1];
      strcpy(hs, s);
      Strings->Add(hs, hs);
      //cout << "New string a: \"" << hs << "\"" << endl << flush;
    }
    return hs;
  }
  return 0;
}

// do strlit on first len characters of s
const char* strnlit(const char* s, int len)
{
  if(s) {
    unsigned int tmp = hash_len;

    hash_len = len;
    char* hs = (char*)Strings->Lookup((char*)s);
    hash_len = tmp;

    if(!hs) {
      hs = new char[len+1];
      strncpy(hs, s, len);
      hs[len] = 0;
      Strings->Add(hs, hs);
      //cout << "New string b: \"" << hs << "\"" << endl << flush;
    }
    return hs;
  }
  return 0;
}

// tmp buffer for string operations
static char _string_buff[2048];

// concat two strings
String String::operator + (const String& s) const
{
  return String(strlit((char*)strcat(strcpy(_string_buff, ptr), s.ptr)));
}

// return substring of string
String String::operator()(int s, int e) const
{
  if(e > s && e < strlen(ptr)) {
    char* p = (char*)(&ptr[e]);
    char c = *p;
    *p = 0;
    String n = &ptr[s];
    *p = c;
    return n;
  }
  return *this;
}

// print string
ostream& operator<<(ostream& os, const String& s)
{
  os << s.ptr;
  return os;
}

// input string from stream
istream& operator>>(istream& is, String& s)
{
  is >> _string_buff;
  s = _string_buff;
  return is;
}


// concat String and a char*
String operator+(const char* s, const String& n)
{
  return String(s) + n;
}

// concat string and a char*
String operator+(const String& s, const char* n)
{
  return s + String(n);
}

// add an integer to a string (integer is converted to a string)
String operator+(const String& n, const int i)
{
  sprintf(_string_buff, "%d", i);
  return n + String(_string_buff);
}

// initialize a string from a char*
String String::Init(const char* s)
{
  return String(rstrlit(s));
}
