#include <search.h>
#include <stream.h>

#ifndef __GNUC__
#include "Vector.h"
#define inline
#endif

// set size of vector and preserve old data
template <class T>
inline void Vector<T>::SetSize(unsigned int new_active, unsigned int n)
{
  if(n == NotFound) n = new_active;
  assert(n >= new_active);
  if(n > _total) {
    T* oldd = _data;
    _data = new T[n];
    for(unsigned int i = 0; i < min(n, _active); i++)
      _data[i] = oldd[i];
    delete[] oldd;
  }
  _total = n;
  _active = new_active;
}

// copy data from another vector
template <class T>
inline void Vector<T>::CopyVector(const Vector<T>& v)
{
  SetSize(v._active);
  for(unsigned int i = 0; i < v._active; i++)
    _data[i] = v._data[i];
}

// append another vector onto this one
template <class T>
inline void Vector<T>::AppendVector(const Vector<T>& v)
{
  unsigned int last = _active;
  Expand(v._active);
  for(unsigned int i = 0; i < v._active; i++)
    _data[last++] = v._data[i];
}

// insert a new element into a vector before/after given position
template <class T>
inline T& Vector<T>::Insert(unsigned int i, T d, Where where)
{
  if(i <= _active) {
    Expand(1);
    if(where == InsertAfter) i++;
    for(unsigned int n = _active-1; n > i; n--)
      _data[n] = _data[n-1];
    return (_data[i] = d);
  }
  return dummy();
}

// remove ith element
template <class T>
inline void Vector<T>::Remove(unsigned int i)
{
  if(i < _active) {
    Shrink(1);
    for(unsigned int n = i; n < _active; n++)
      _data[n] = _data[n+1];
  }
}

// compare two vectors for equality
template <class T>
inline Boolean Vector<T>::operator==(const Vector<T>& l) const
{
  if(l._active == _active) {
    for(unsigned int i = 0; i < _active && _data[i] == l._data[i]; i++);
    return MakeBoolean(i == _active);
  }
  return False;
}

// vector inequality
template <class T>
inline Boolean Vector<T>::operator!=(const Vector<T>& l) const
{
  if(l._active == _active) {
    for(unsigned int i = 0; i < _active && _data[i] == l._data[i]; i++);
    return MakeBoolean(i != _active);
  }
  return True;
}

// print a vector
template <class T>
inline ostream& Vector<T>::Print(ostream& os,int) const
{
  os << "Vector<T>(" << this << ", " << _active << ", " << _total << ")";
  if(_active) {
    os << endl;
    for(unsigned int i = 0; i < _active; i++) {
      os << "[" << i << "] = " << _data[i];
      if(i != _active - 1) os << endl;
    }
  }
  return os;
}
