#include <assert.h>
#include <stream.h>

#ifdef USE_FREELIST
#include "FreeList.h"

// increase number of elements in free list
void FreeList::Expand(int grow, void* p)
{
  // placement operator seems busted
  //FreeListElem* new_chunk = (FreeListElem*)::operator new((_obj_size+1)*grow, p);
  FreeListElem* new_chunk = (FreeListElem*)::operator new((_obj_size+1)*grow);
  FreeListElem* tmp = (FreeListElem*)DataAddr(new_chunk);

  assert(tmp);

  _num_chunks++;
  if(!_chunks) {
    _chunks = new_chunk;
    _chunks->_next = 0;
  } else {
    new_chunk->_next = _chunks;
    _chunks = new_chunk;
  }

  //cout << "expand size = " << grow*_obj_size << ", " << place << endl;

  // could do this on the fly to save time
  for(unsigned int i = 0; i < grow; i++) {
    tmp->_next = _cache;
    _cache = tmp;
    tmp = (FreeListElem*)((char*)tmp + _obj_size);
  }
  _size += grow;
  _total += grow;
}

// remove an element from the free list
// won't work for array allocation!
void* FreeList::Alloc(size_t s, void* p)
{
  _num_allocs++;

  if(!_cache && (!_limit || _total < _limit))
    Expand(_expand, p);

  if(_cache) {
    // apply p somehow?
    return Pop();
  } else {
    // busted on stupid CC compiler. argh!!!
    //return DataAddr(::operator new(_obj_size, p));
    return DataAddr(::operator new(_obj_size));
  }
}

// create a new freelist
FreeList::FreeList(char* class_name, int obj_size,
		   Boolean verbose, int init,
		   int limit, int exp, void* p)
: _class_name(Strdup(class_name)), _verbose(verbose),
  _obj_size(obj_size + sizeof(FreeListElem)),
  _cache(0), _place(p), _size(0), _total(0),
  _limit(limit), _num_allocs(0), _num_frees(0),
  _num_chunks(0), _chunks(0)
{
  _expand = exp ? exp : 1000;
  if(init) Expand(init, _place);
}

// destroy a freelist
FreeList::~FreeList()
{
  if(_verbose) {
    cout << _class_name << ", Allocs/Frees/Chunks = " << _num_allocs << "/";
    cout << _num_frees << "/" << _num_chunks << endl;
  }
  delete _chunks;
}

#endif
