#include <iomanip.h>
#include "String.h"
#include "Object.h"
#include "Vector.h"

// brain damage! this is defined in String.h but compiler croaks!
extern const char* rstrlit(const char*);

// don't ask
#ifdef STUPID_TEMPLATES
#include "Vector.h"
#define superclasses ((Vector<ClassInfo*>*)_superclasses)
#define subclasses ((Vector<ClassInfo*>*)_subclasses)
#define all_classes ((Vector<ClassInfo*>*)ClassInfo::_all_classes)
void* ClassInfo::_all_classes = 0;
#else
#define superclasses _superclasses
#define subclasses _subclasses
#define all_classes ClassInfo::_all_classes
Vector<ClassInfo*>* ClassInfo::_all_classes = 0;
#endif

// more compiler stupidity
#ifndef __GNUC__
Boolean operator == (const ClassInfo* a, const ClassInfo* b)
{
  return MakeBoolean(a->class_name == b->class_name);
}

Boolean operator > (const ClassInfo* a, const ClassInfo* b)
{
  return MakeBoolean(strcmp(a->class_name, b->class_name) > 0);
}

Boolean operator < (const ClassInfo* a, const ClassInfo* b)
{
  return MakeBoolean(strcmp(a->class_name, b->class_name) < 0);
}
#endif

// IsA operation for classes
Boolean ClassInfo::IsA(ClassInfo* ci) const
{
  if(ci == this)
    return True;

  if(_superclasses) {
    for(unsigned int i = 0; i < superclasses->Size(); i++)
      if((*superclasses)[i]->IsA(ci))
	return True;
  }

  return False;
}

// is operation from ClassId only
Boolean ClassInfo::IsA(ClassId id) const
{
  return id ? IsA((*all_classes)[id-1]) : False;
}

// create a new class info for given class
ClassInfo* ClassInfo::Create(const char* name, ...)
{
  va_list args;
  va_start(args, name);

  if(!_all_classes)
    _all_classes = new Vector<ClassInfo*>;

#ifndef __GNUC__
  ClassInfo dummy(name, 0);
  ClassInfo* ci = all_classes->Find(&dummy);
#else
  ClassInfo* ci = 0;
  name = rstrlit(name);
  for(int i = 0; i < all_classes->Size(); i++) {
    if((*all_classes)[i]->class_name == name) {
      ci = (*all_classes)[i];
      break;
    }
  }
#endif

  if(!ci) {
    ci = new ClassInfo(name, all_classes->Size()+1);
    all_classes->Append(ci);
  }

  ci->AddSuperClasses(args);

  va_end(args);

  return ci;
}

// add list of superclasses to class info
void ClassInfo::AddSuperClasses(va_list args)
{
  const char* cname = va_arg(args, char*);
  while(cname && *cname != '0') {
    ClassInfo* pci = ClassInfo::Create(cname, 0);
    pci->AddSubClass(this);
    AddSuperClass(pci);
    cname = va_arg(args, char*);
  }
}

// add single superclass to class info
void ClassInfo::AddSuperClass(ClassInfo* ci)
{
  superclasses->AppendNew(ci);
}

// add single subclass to class info
void ClassInfo::AddSubClass(ClassInfo* ci)
{
  subclasses->AppendNew(ci);
}

// create a new class info
ClassInfo::ClassInfo(const char* cname, ClassId id)
: class_id(id), class_name(rstrlit(cname)),
  _superclasses(new Vector<ClassInfo*>),
  _subclasses(new Vector<ClassInfo*>)
{
}

// destroy a class info
ClassInfo::~ClassInfo()
{
  delete superclasses;
  delete subclasses;
}

// print a class info
ostream& ClassInfo::Print(ostream& os, int) const
{
  os << "class " << class_name;
  if(superclasses->Size()) {
    os << " : ";
    for(int i = 0; i < superclasses->Size(); i++) {
      os << "public " << (*superclasses)[i]->class_name;
      if(i != superclasses->Size() - 1) os << ", ";
    }
  }
  return os;
}

// print entire class hierarchy
ostream& ClassInfo::PrintAll(ostream& os)
{
  if(all_classes) {
    for(int i = 0; i < all_classes->Size(); i++)
      os << *(*all_classes)[i] << ";" << endl;
  }
  return os;
}

/////////////////////////////////////////////////////////////////////////////
//
// SimpleObject & Object
//
/////////////////////////////////////////////////////////////////////////////

// print a simple object
ostream& operator<<(ostream& os, const SimpleObject& obj)
{
  return obj.Print(os);
}

DefineStandardMethods(Object,0);

// print an Object
ostream& Object::Print(ostream& os, int) const
{
  os << setiosflags(ios::hex);
  os << "#<" << ClassName() << " " << hex << (void*)this << ">";
  os << resetiosflags(ios::hex);
  return os;
}
