////////////////////////////////////////////////////////////////////////////
//
// Object.h - common superclass which provides MOP info
//
////////////////////////////////////////////////////////////////////////////
#ifndef _OBJECT_H_
#define _OBJECT_H_

#include <stdarg.h>
#include "SimpleObject.h"

template <class T> class Vector;

typedef unsigned int ClassId;

// MOP info for a class
class ClassInfo : public SimpleObject {
 public:
  const char* ClassName() const { return class_name; }
  ClassId Id() const { return class_id; }
  Boolean IsA(ClassInfo*) const;
  Boolean IsA(ClassId) const;
  static ClassInfo* Create(const char*, ...);
  ostream& Print(ostream&,int=0) const;
  static ostream& PrintAll(ostream&);
#ifndef __GNUC__
  friend Boolean operator == (const ClassInfo*, const ClassInfo*);
  friend Boolean operator > (const ClassInfo*, const ClassInfo*);
  friend Boolean operator < (const ClassInfo*, const ClassInfo*);
#endif

 protected:
  void AddSuperClass(ClassInfo*);
  void AddSubClass(ClassInfo*);
  void AddSuperClasses(va_list);
  ClassInfo(const char*, ClassId);
  ~ClassInfo();

 private:
  ClassId class_id;
  const char* class_name;
#ifdef STUPID_TEMPLATES
  void* _superclasses;
  void* _subclasses;
  static void* _all_classes;
#else
  Vector<ClassInfo*>* _superclasses;
  Vector<ClassInfo*>* _subclasses;
  static Vector<ClassInfo*>* _all_classes;
#endif
};

#define DeclareStandardMethodsNoPrint(cname, parent) \
 public: \
  virtual const char* ClassName() const { assert(cname::_ci_()); return cname::_ci_()->ClassName(); } \
  virtual ClassId ObjectId() const { assert(cname::_ci_()); return cname::_ci_()->Id(); } \
  virtual Boolean IsA(ClassId x) const { assert(cname::_ci_()); return cname::_ci_()->IsA(x); } \
  static ClassId Id() { assert(cname::_ci_()); return cname::_ci_()->Id(); } \
 private: \
  static ClassInfo* _ci_()\
  {\
    static ClassInfo* pci = ClassInfo::Create(#cname, #parent, 0);\
    return pci;\
  }\
  friend class __stupid__

#define DeclareStandardMethods(cname, parent) \
  DeclareStandardMethodsNoPrint(cname, parent);\
 public: \
  virtual ostream& Print(ostream&, int = 0) const


// following macros are not required to be used.  They just insure
// that all class info's are created at the time the program starts.

#define DefineStandardMethods(cname, parent)\
static int cname##dummy = cname::Id()

#define DefineStandardMethods2(cname, p1, p2)\
static int cname##dummy = cname::Id()

// can't do it for templates
#define DefineStandardTemplateMethods(t, cname, parent)
#define DefineStandardTemplateMethods2(t, cname, p1, p2)

class Object : public SimpleObject {
  DeclareStandardMethods(Object,0);
};

#endif
