/* This file implements functions for simple timer objects.
 * These timers are used to measure intervals of time (like for
 * performance measuring).  Each timer will record the total amount
 * of time it has run, the average interval over all the times it has
 * run and the largest and smallest intervals over which it has run.
 * All times  are recorded in terms of micro seconds.  The routines
 * provided in this file allow the user to create new timers, start
 * and stop existing timers and query the attributes of a timer.
 */
#ifndef _TIMER_H_
#define _TIMER_H_

typedef int tid_t;

extern tid_t timer_create(const char*);
extern void timer_start(tid_t);
extern void timer_stop(tid_t);
extern void timer_print(tid_t);
extern void timer_print_all(void);
extern int timer_is_on(tid_t);
extern float timer_delta(tid_t);
extern float timer_total(tid_t);
extern float timer_max(tid_t);
extern float timer_min(tid_t);
extern int timer_num(tid_t);

/* maximum number of timers */
#define MAX_TIMERS   1000

#endif
