////////////////////////////////////////////////////////////////////////////
//
// tsearch.h - templatized bsearch and lsearch
//
////////////////////////////////////////////////////////////////////////////
#ifndef _TSEARCH_H_
#define _TSEARCH_H_

#include "misc.h"

// templatized binary search
template <class T>
inline unsigned int bsearch_index(T& val, T* data,
				  unsigned int num,
				  Boolean must_exist = True)
{
  unsigned int m = 0;
  int res = -1;

  if(num) {
    unsigned int u = num - 1;
    unsigned int l = 0;
    m = (l+u)>>1;
    res = (val < data[m] ? -1 : (val == data[m] ? 0 : 1));
    while(res && l < u) {
      if(res > 0)
	l = m + 1;
      else if(res < 0)
	u = m ? m - 1 : l++, 0;
      else
	break;
      m = (l + u) >> 1;
      res = (val < data[m] ? -1 : (val == data[m] ? 0 : 1));
    }
    if(res > 0) m++;
  }
  return (!must_exist || !res) ? m : NotFound;
}

// templatized linear search
template <class T>
inline unsigned int lsearch_index(T& val, T* data, unsigned int num)
{
  for(unsigned int i = 0; i < num; i++)
    if(val == data[i])
      return i;
  return NotFound;
}

#endif
