#include <stream.h>
#include <ctype.h>
#include <stdarg.h>
#include <stdio.h>
#include "dx++.h"
#include "dxfield.h"

typedef DXObject (*DXExtractFn)(char**, DXObject, Type, Class, int, int, void*);

// extract a byte from a DXObject
DXObject _DXExtractByte(char**, DXObject o, Type, Class, int, int, unsigned char* arg)
{
  int tmp;
  DXObject otmp = DXExtractInteger(o, &tmp);
  *arg = (unsigned char)tmp;
#ifdef DEBUG
  cout << *arg;
#endif
  return otmp;
}

// extract a short from a DXObject
DXObject _DXExtractShort(char**, DXObject o, Type, Class, int, int, unsigned short* arg)
{
  int tmp;
  DXObject otmp = DXExtractInteger(o, &tmp);
  *arg = (unsigned short)tmp;
#ifdef DEBUG
  cout << *arg;
#endif
  return otmp;
}

// extract an int from a DXObject
DXObject _DXExtractInt(char**, DXObject o, Type, Class, int, int, unsigned int* arg)
{
  int tmp;
  DXObject otmp = DXExtractInteger(o, &tmp);
  *arg = (unsigned int)tmp;
#ifdef DEBUG
  cout << *arg;
#endif
  return otmp;
}

// extract a long from a DXObject
DXObject _DXExtractLong(char**, DXObject o, Type, Class, int, int, unsigned long* arg)
{
  int tmp;
  DXObject otmp = DXExtractInteger(o, &tmp);
  *arg = (unsigned long)tmp;
#ifdef DEBUG
  cout << *arg;
#endif
  return otmp;
}

// extract a float from a DXObject
DXObject _DXExtractFloat(char**, DXObject o, Type, Class, int, int, float* arg)
{
  float tmp;
  DXObject otmp = DXExtractFloat(o, &tmp);
  *arg = tmp;
#ifdef DEBUG
  cout << *arg;
#endif
  return otmp;
}

// extract a double from a DXObject
DXObject _DXExtractDouble(char**, DXObject o, Type, Class, int, int, double* arg)
{
  float tmp;
  DXObject otmp = DXExtractFloat(o, &tmp);
  *arg = (double)tmp;
#ifdef DEBUG
  cout << *arg;
#endif
  return otmp;
}

// extract a string from a DXObject
DXObject _DXExtractString(char**, DXObject o, Type, Class, int, int, char** arg)
{
  char* tmp = 0;
  DXObject otmp = DXExtractString(o, &tmp);
  *arg = tmp;
#ifdef DEBUG
  cout << *arg;
#endif
  return otmp;
}

// extract a field from a DXObject and convert it to a DXField
DXObject _DXExtractField(char**, DXObject o, Type, Class, int, int, DXField* arg)
{
  if(DXGetObjectClass(o) != CLASS_FIELD)
    return 0;
  arg->Init(o);
#ifdef DEBUG
  cout << *arg;
#endif
  return o;
}

// extract generic DXObject from a DXObject
DXObject _DXExtractObject(char**, DXObject o, Type, Class c, int, int, DXObject* arg)
{
  if(c && DXGetObjectClass(o) != c)
    return 0;
  *arg = o;
#ifdef DEBUG
  cout << *arg;
#endif
  return o;
}

// extract a Point3d from a DXObject
DXObject _DXExtractPoint(char**, DXObject o, Type, Class, int, int, Point3d* arg)
{
  Class oc = DXGetObjectClass(o);

  if(oc != CLASS_ARRAY && oc != CLASS_REGULARARRAY && oc != CLASS_CONSTANTARRAY)
    return 0;

  int size, r, s;
  Category cat;
  Type t;

  DXGetArrayInfo((Array)o, &size, &t, &cat, &r, &s);
  if(size != 1 || cat != CATEGORY_REAL || r != 1 || s != 3)
    return 0;

  if(t == TYPE_FLOAT) {
    *arg = *(Point3d*)DXGetArrayData((Array)o);
  } else if(t == TYPE_INT) {
    int* data = (int*)DXGetArrayData((Array)o);
    *arg = Point3d(data[0], data[1], data[2]);
  } else {
    return 0;
  }

#ifdef DEBUG
  cout << *arg;
#endif
  return o;
}

// table of extraction formats and functions (for dxscanf)
struct {
  char* _Fmt;
  Type _Type;
  Class _Class;
  DXExtractFn _Extract;
} formats[] = {
  { "c", TYPE_BYTE, (Class)0, _DXExtractByte },
  { "uc", TYPE_UBYTE, (Class)0, _DXExtractByte },
  { "hd", TYPE_SHORT, (Class)0, _DXExtractShort },
  { "hu", TYPE_USHORT, (Class)0, _DXExtractShort },
  { "d", TYPE_INT, (Class)0, _DXExtractInt },
  { "u", TYPE_UINT, (Class)0, _DXExtractInt },
  { "l", TYPE_INT, (Class)0, _DXExtractLong },
  { "ul", TYPE_UINT, (Class)0, _DXExtractLong },
  { "f", TYPE_FLOAT, (Class)0, _DXExtractFloat },
  { "e", TYPE_DOUBLE, (Class)0, _DXExtractDouble },
  { "g", TYPE_DOUBLE, (Class)0, _DXExtractDouble },
  { "s", TYPE_STRING, (Class)0, _DXExtractString },
  { "F", (Type)0, CLASS_FIELD, _DXExtractField },
  { "A", (Type)0, CLASS_ARRAY, _DXExtractObject },
  { "L", (Type)0, CLASS_LIGHT, _DXExtractObject },
  { "C", (Type)0, CLASS_CAMERA, _DXExtractObject },
  { "O", (Type)0, (Class)0, _DXExtractObject },
  { "P", (Type)0, (Class)0, _DXExtractPoint },
  { "V", (Type)0, (Class)0, _DXExtractPoint },
  { 0, (Type)0, (Class)0, 0 },
};

// like sscanf except it takes input from an array of DXObjects
// see format table for available types
Error dxscanf(DXObject* in, char* format, ...)
{
  static char forms[256];
  char* p = format;
  char* fptr = 0;
  int index = 0;
  va_list args;
  void* arg = 0;
  Boolean required = False;
  int dim = 0;
  int count = 1;
  Error error = OK;

  va_start(args, format);

  do {
    arg = va_arg(args, void*);

    fptr = forms;
    *fptr++ = *p;
    if(*p++ != '%')
      DXErrorReturn(ERROR_INVALID_DATA, "dxscanf: bad format string");

    *fptr = *p;
    if(*p == 'r') {
      required = True; p++; fptr++;
    } else {
      required = False;
    }

#if 0
    if(isdigit(*p)) {
      sscanf(p, "%d", &dim);
      while(isdigit(*p++));
    }

    if(*p == '.') {
      sscanf(++p, "%d", &dim);
      while(isdigit(*p++));
    }

    if(*p == '*' || *p == 'n') {
      p++;
    }
#endif

    char* f = fptr;
    while(*p && isalpha(*p))
      *fptr++ = *p++;
    *fptr++ = 0;

    for(int i = 0; formats[i]._Fmt; i++) {
      if(!strcmp(f, formats[i]._Fmt)) {
	DXObject o = in[index];
	Type t = formats[i]._Type;
	Class c = formats[i]._Class;

#ifdef DEBUG
	cout << "Extract arg " << index << " as " << forms << " ";
#endif
	if(o) {
	  if(!formats[i]._Extract(0, o, t, c, 0, 0, arg)) {
#ifdef DEBUG
	    cout << " failed!" << endl;
#endif
	    DXSetError(ERROR_BAD_PARAMETER, "dxscanf: type mismatch at arg %d(%s)", index, forms);
	    error = ERROR;
	  } else {
#ifdef DEBUG
	    cout << " ok." << endl;
#endif
	  }
	} else if(required) {
	  DXSetError(ERROR_BAD_PARAMETER, "dxscanf: required arg %d missing", index);
	  error = ERROR;
	} else {
#ifdef DEBUG
	  cout << "skipped (using default)." << endl;
#endif
	}
	break;
      }
    }
    if(!formats[i]._Fmt) {
      DXSetError(ERROR_BAD_PARAMETER, "dxscanf: unknown format at arg %d(%s)", index, forms);
      error = ERROR;
    }

    index++;
  } while(*p && error == OK);

  va_end(args);

  return error;
}

// convert a single floating point value into a DXObject
DXObject ToDX(float r)
{
  FloatArray a(1);
  a[0] = r;
  return a;
}

// print a DX RGBColor object
ostream& operator<<(ostream& os, const RGBColor& c)
{
  os << "#<RGBColor " << c.r << ", " << c.g << ", " << c.b << ">";
  return os;
}

// transpose data in an array (since DX seems to be column major)
void Transpose(float* data, int w, int h, int d)
{
  float* tmp = new float[w*h*d];
  int n = 0;
  for(int i = 0; i < w; i++)
    for(int j = 0; j < h; j++)
      tmp[n++] = data[i + j * w];
  MemCopy(tmp, data, w*h*d*sizeof(*data));
  delete[] tmp;
}

