#include <stream.h>
#include "dxarray.h"

// copy an array (this one does nothing since we can't know the array type)
DXArrayBase* DXArrayBase::Copy() const
{
  return 0;
}

// set size of an array
void DXArrayBase::SetSize(int s)
{
  DXAddArrayData(_a, 0, s, 0);
  reset_handle();
}

// add data to an array
void DXArrayBase::Grow(int amt)
{
  DXAddArrayData(_a, Size(), amt, 0);
  reset_handle();
}

// print an array
ostream& DXArrayBase::Print(ostream& os, int v) const
{
  if(_a) {
    os << "DXArray: " << ArrayClass() << "/" << EltType() << "/";
    os << EltCategory() << "/" << Size() << "/" << Rank() << "(";
    for(int i = 0; i < Rank(); i++) {
      os << Shape(i);
      if(i != Rank() - 1) os << ", ";
    }
    os << ")";
    if(v) {
      os << endl;
      for(i = 0; i < Size(); i++) {
	char* data = (char*)Ptr(i);
	os << "data[" << i << "] = ";
	for(int j = 0; j < Rank(); j++) 
	  for(int k = 0; k < Shape(j); k++)
	    ePrint(os, &data[(j * Shape(j) + k) * EltSize()]);
	os << endl;
      }
    }
  } else {
    os << "DXArray: <empty>";
  }
  return os;
}

// get the class of an array
Class DXArrayBase::ArrayClass() const
{
  return _a ? DXGetArrayClass(_a) : (Class)0;
}

// get the element type of an array
Type DXArrayBase::EltType() const
{
  Type t = (Type)0;
  if(_a) DXGetArrayInfo(_a, 0, &t, 0, 0, 0);
  return t;
}

// get the element category of an array
Category DXArrayBase::EltCategory() const
{
  Category cat = (Category)0;
  if(_a) DXGetArrayInfo(_a, 0, 0, &cat, 0, 0);
  return cat;
}

// get the element size of an array
int DXArrayBase::EltSize() const
{
  return _a ? DXGetItemSize(_a) : 0;
}

// get the size of an array
int DXArrayBase::Size() const
{
  int s = 0;
  if(_a) DXGetArrayInfo(_a, &s, 0, 0, 0, 0);
  return s;
}

// get the rank of an array
int DXArrayBase::Rank() const
{
  int r = 0;
  if(_a) DXGetArrayInfo(_a, 0, 0, 0, &r, 0);
  return r;
}

// return the shape table for this array (rank elements long)
int* DXArrayBase::ShapeTable() const
{
  static int last_rank = 0;
  static int* s = 0;
  int rank = Rank();
  if(rank > last_rank) {
    if(s) delete[] s;
    s = new int[rank];
    last_rank = rank;
  }
  if(_a) DXGetArrayInfo(_a, 0, 0, 0, &rank, s);
  return s;
}

// return the shape of the nth rank
int DXArrayBase::Shape(int n) const
{
  return _a ? ShapeTable()[n] : 0;
}

// reset the ArrayHandle for this array
void DXArrayBase::reset_handle()
{
  if(_a) {
    DXFreeArrayHandle(_h);
    _h = DXCreateArrayHandle(_a);
  }
}

// create an empty array
DXArrayBase::DXArrayBase()
: _a(0), _h(0), _own(False)
{
}

// create an array from a real DX Array
DXArrayBase::DXArrayBase(Array a)
: _a(a), _h(a ? DXCreateArrayHandle(a) : 0), _own(False)
{
}

// copy a DXArray wrapper
DXArrayBase::DXArrayBase(const DXArrayBase& a)
: _a(a._a), _h(a._a ? DXCreateArrayHandle(a._a) : 0), _own(False)
{
}

// destroy an array
// (destroy the ArrayHandle and the array data if it is owned)
DXArrayBase::~DXArrayBase()
{
  if(_h) DXFreeArrayHandle(_h);
  if(_a && _own) DXDelete((DXObject)_a);
}
