////////////////////////////////////////////////////////////////////////////
//
// dxarray.h - templatized C++ wrapper for DX Array object
//
////////////////////////////////////////////////////////////////////////////
#ifndef _DXARRAY_H_
#define _DXARRAY_H_

#include <stream.h>
#include "SimpleObject.h"
#include "dx++.h"

// abstract base class for Array wrapper
class DXArrayBase : public SimpleObject {
 public:
  // copy this array
  virtual DXArrayBase* Copy() const;

  // convert back into a DX Object, set own to False since we don't
  // know what DX is going to do with this array
  DXObject ToDX() const { own() = False; return (DXObject)_a; }

  // convert back into a DX Array
  operator Array() const { return (Array)ToDX(); }

  // print an array
  ostream& Print(ostream& os, int v=0) const;

  // return pointer to array data
  void* Ptr() const { return DXGetArrayData(_a); }

  // return pointer to ith array element
  void* Ptr(int i) const
    { static char crap[256]; return DXGetArrayEntry(_h, i, crap); }

  void SetSize(int s);           // set array size
  void Grow(int amt);            // grow array by amt elements
  Class ArrayClass() const;      // get the array class
  Type EltType() const;          // get the element type
  Category EltCategory() const;  // get the element category
  int EltSize() const;           // get the element size in bytes
  int Size() const;              // get size of this array
  int Rank() const;              // get rank of this array
  int* ShapeTable() const;       // get shape table for this array
  int Shape(int n = 0) const;    // return shape for nth rank

  // is this array empty?
  Boolean IsEmpty() const { return MakeBoolean(!Size()); }

  // get own flag
  Boolean Own() const { return _own; }

  DXArrayBase();                       // new empty array
  DXArrayBase(Array a);                // new wrapper around Array a
  DXArrayBase(const DXArrayBase& a);   // copy array wrapper
  ~DXArrayBase();                      // destroy array wrapper
 protected:
  // create a new ArrayHandle for this array (and delete the old one)
  void reset_handle();

  // access to private data
  Array array() const { return ((DXArrayBase*)this)->_a; }
  ArrayHandle array_handle() const { return ((DXArrayBase*)this)->_h; }
  Boolean& own() const { return ((DXArrayBase*)this)->_own; }

  // print an array element (so print looks nice)
  virtual void ePrint(ostream& os, void* d) const { os << d; }
 private:
  Array _a;        // actual DX Array object
  ArrayHandle _h;  // array handle used to access data
  Boolean _own;    // own flag for Array data
};

// templatized subclass of DXArrayBase, allows
// type correct access elements of an array
template <class T> class DXArray : public DXArrayBase {
 public:
  T* Data() const { return (T*)Ptr(); }
  T& operator[](int i) const { return *(T*)Ptr(i); }
  T* operator()(int i) const { return (T*)Ptr(i); }
  DXArray() : DXArrayBase() { }
  DXArray(Array a) : DXArrayBase(a) { }
  DXArray(const DXArray<T>& a) : DXArrayBase(a) { }
 protected:
  void ePrint(ostream& os, void* d) const { os << *(T*)d; }
};

#endif
