#include "dxcolor.h"
#include "palette.h"

static int ColorMapSize(Field f)
{
  Array a = (Array)DXGetComponentValue(f, "data");
  int size;
  DXGetArrayInfo(a, &size, 0, 0, 0, 0);
  return size;
}

ColorMap::operator DXObject() const
{
  DXSetComponentValue(_cmap, "data", _color);
  DXSetComponentValue(_cmap, "positions", _index);
  DXChangedComponentStructure(_cmap, "data");
  DXChangedComponentStructure(_cmap, "positions");

  DXSetComponentValue(_cmap, "connections", (DXObject)DXMakeGridConnections(1, Size()));
  DXChangedComponentStructure(_cmap, "connections");

  DXEndField(_cmap);
  return (DXObject)_cmap;
}


ColorMap::ColorMap(int size)
: _cmap(DXNewField()), _color(size), _index(size)
{
  DXSetComponentValue(_cmap, "connections", (DXObject)DXMakeGridConnections(1, size));
  DXChangedComponentStructure(_cmap, "connections");
}

ColorMap::ColorMap(Field cmap)
: _cmap((Field)DXCopy((DXObject)cmap, COPY_STRUCTURE)),
  _color(ColorMapSize(cmap)), _index(ColorMapSize(cmap))
{
  ColorArray oldc((Array)DXGetComponentValue(cmap, "data"));
  FloatArray oldi((Array)DXGetComponentValue(cmap, "positions"));
  for(int i = 0; i < Size(); i++) {
    _color[i] = oldc[i];
    _index[i] = oldi[i];
  }
}

ColorMap::ColorMap(const ColorMap& c)
: _cmap((Field)DXCopy((DXObject)c._cmap, COPY_STRUCTURE)),
  _color(c.Size()), _index(c.Size())
{
  for(int i = 0; i < Size(); i++) {
    _color[i] = c._color[i];
    _index[i] = c._index[i];
  }
}

ColorMap::ColorMap(const Palette& p, float min, float max)
: _cmap(DXNewField()), _color(p.Size()), _index(p.Size())
{
  for(int i = 0; i < Size(); i++) {
    RGB rgb = (RGB&)p[i];
    _color[i] = DXRGB(rgb.r(), rgb.g(), rgb.b());
    _index[i] = (((max - min) * i) / (p.Size() - 1)) + min;
  }
  DXSetComponentValue(_cmap, "connections", (DXObject)DXMakeGridConnections(1, p.Size()));
  DXSetComponentAttribute(_cmap, "connections", "ref", (DXObject)DXNewString("positions"));
  DXChangedComponentStructure(_cmap, "connections");
}

ColorMap::ColorMap(Field f, const Palette& p, float min, float max)
: _cmap((Field)DXCopy((DXObject)f, COPY_STRUCTURE)),
  _color(p.Size()), _index(p.Size())
{
  for(int i = 0; i < Size(); i++) {
    RGB rgb = (RGB&)p[i];
    _color[i] = DXRGB(rgb.r(), rgb.g(), rgb.b());
    _index[i] = (((max - min) * i) / (p.Size() - 1)) + min;
  }
  DXSetComponentValue(_cmap, "connections", (DXObject)DXMakeGridConnections(1, p.Size()));
  DXSetComponentAttribute(_cmap, "connections", "ref", (DXObject)DXNewString("positions"));
  DXChangedComponentStructure(_cmap, "connections");
}
