#include "dxfield.h"

#if 0  //junk
class ComponentInfo {
 public:
  void Update(DXObject o) {
    String sname = name;
    ArrayHandle h = _hdl.Lookup(sname);
    if(h) DXFreeArrayHandle(h);
    Array a = (Array)GetComponent(sname);
    h = DXCreateArrayHandle(a);
    _hdl.Set(sname, h)
    }

  ComponentInfo();
  ComponentInfo(DXObject o);
  ~ComponentInfo() { DXFreeArrayHandle(_hdl); }

  Boolean _modified;
  ArrayHandle _hdl;
  int _size;
  int _rank;
  int _shape;
};
#endif

// Turn DXField back into Field, this consists of copying the
// positions and data out of the local field object into the
// DX object and calling DXEndField.
// put error checking in here
// add an ERROR field to DXField which should be set if any dx
// op has an error.  if ERROR is set when ToField is called then
// return 0, which can be checked for error.
Field DXField::ToField() const
{
  ((DXField*)this)->_own = False;
  ((DXField*)this)->set_field_positions();
  ((DXField*)this)->set_field_colors();

  if(DXEndField(_field))
    return _field;
  else
    return 0;
}

// set a named component of a field, if mark = True mark the
// component as changed.  When setting the colors or positions
// components copy them into the local colors or components arrays
void DXField::SetComponent(char* name, DXObject o, Boolean mark)
{
  DXSetComponentValue(_field, name, o);
  if(!strcmp(name, "colors")) {
    if(!color()) set_colors(new RGB[Size()]);
    get_field_colors();
  } else if(!strcmp(name, "positions")) {
    if(!data()) set_data(new Point3d[Size()]);
    get_field_positions();
  }
  if(mark) DXChangedComponentValues(_field, name);
}

// set array component
void DXField::SetComponent(char* name, Array a, Boolean mark)
{
  SetComponent(name, (DXObject)a, mark);
}

// set a component attribute from a string
void DXField::SetComponentAttribute(char* name, char* attr, char* val)
{
  SetComponentAttribute(name, attr, (DXObject)DXNewString(val));
}

// set a component attribute
void DXField::SetComponentAttribute(char* name, char* attr, DXObject val)
{
  DXSetComponentAttribute(_field, name, attr, val);
}

// get a named component
DXObject DXField::GetComponent(char* name) const
{
  return DXGetComponentValue(_field, name);
}

// get an array component.
// if the component is a point array or a single value float
// array then return the exact type, otherwise just return
// a generic array wrapper
DXArrayBase DXField::GetArrayComponent(char* name) const
{
  Array a = (Array)GetComponent(name);
  if(a) {
    int rank, shape[256];
    Type t;
    Category c;
    DXGetArrayInfo(a, 0, &t, &c, &rank, shape);
    if(t == TYPE_FLOAT && c == CATEGORY_REAL && rank == 1) {
      if(shape[0] == 3)
	return PointArray(a);
      else if(shape[0] == 1)
	return FloatArray(a);
    }
  }
  return DXArrayBase(a);
}

// initialize local field data from DX field.
// first determine the dimensions of the field and
// then copy the positions (and possible colors)
// components into the local arrays
void DXField::Init(Field f)
{
  int w, h, d;

  _field = (Field)DXCopy((DXObject)f, COPY_STRUCTURE);
  DXASSERT(_field);

  compute_field_size(w, h, d);
  MyField::set_size(w, h, d, 1.0);

  MyField::set_data(new Point3d[Size()]);
  get_field_positions();

  if(DXExists((DXObject)_field, "colors")) {
    MyField::set_colors(new RGB[Size()]);
    get_field_colors();
  }

  own() = True;
  _own = True;
}

// create a new empty field
DXField::DXField()
: MyField(), _own(False), _field(0)
{
}

// create a field by copying another field
// (see hmap.h for meanings of flags)
DXField::DXField(const MyField& f, Boolean copy, Boolean own)
: MyField(f, copy, own), _own(True), _field(DXNewField())
{
}

// create a field by copying another field
// (see hmap.h for meanings of flags)
DXField::DXField(const DXField& f, Boolean copy, Boolean own)
: MyField(f, copy, own), _own(True),
  _field((Field)DXCopy((DXObject)f._field, COPY_STRUCTURE))
{
}

// create a new field of the given size and optionally
// copy the data from the given field f
DXField::DXField(int w, int h, int d, Field f)
: MyField(w, h), _own(True),
  _field(f ? (Field)DXCopy((DXObject)f, COPY_STRUCTURE) : DXNewField())
{
  DXASSERT(_field);
  if(f) {
    int w, h, d;
    compute_field_size(w, h, d);
    if(w == Width() && h == Height() && d == Depth()) {
      get_field_positions();
      get_field_colors();
    } else {
      cerr << "specified field dimensions do not match DX field dimensions!" << endl;
    }
  }
}

// destroy a field
DXField::~DXField()
{
  if(_own) DXDelete((DXObject)_field);
}


// ***********should add depth!!!***********

// compute dimensions of a field
// assumes a regular grid (otherwise things will be screwy)
// could probably use connections component to determine size!
void DXField::compute_field_size(int& w, int& h, int& d)
{
  PointArray pa = (PointArray&)GetArrayComponent("positions");
  Real lasty = pa[0].y;

  w = h = 0; d = 1;
  while(lasty != pa[++h].y);
  w = pa.Size() / h;
#ifdef DEBUG
//  cout << "Width: " << w << endl;
//  cout << "Height: " << h << endl << flush;
//  cout << "Depth: " << d << endl << flush;
#endif
}

// copy positions component into local array (also transpose data)
void DXField::get_field_positions()
{
  PointArray tmp = (PointArray&)GetArrayComponent("positions");
  if(!tmp.IsEmpty()) {
    int k = 0;
    for(int i = 0; i < Width(); i++)
      for(int j = 0; j < Height(); j++)
	(*this)(i, j) = tmp[k++];
  }
}

// set real field positions component from local data (and transpose)
void DXField::set_field_positions()
{
  PointArray pa(Size());
  int k = 0;
  for(int i = 0; i < Width(); i++)
    for(int j = 0; j < Height(); j++)
      pa[k++] = (*this)(i, j);
  SetComponent("positions", pa, True);
}

// same as get_field_positions() except for colors
void DXField::get_field_colors()
{
  ColorArray tmp = (ColorArray&)GetArrayComponent("colors");
  if(!tmp.IsEmpty()) {
    int k = 0;
    for(int i = 0; i < Width(); i++) {
      for(int j = 0; j < Height(); j++) {
	RGBColor rgb = tmp[k++];
	color(i, j) = RGB(rgb.r, rgb.g, rgb.b);
      }
    }
  }
}

// same as set_field_positions() except for colors
void DXField::set_field_colors()
{
  if(color()) {
    ColorArray colors(Size());
    int k = 0;
    for(int i = 0; i < Width(); i++) {
      for(int j = 0; j < Height(); j++) {
	RGB rgb = (RGB&)color(i, j);
	colors[k++] = DXRGB(rgb.r(), rgb.g(), rgb.b());
      }
    }
    SetComponent("colors", colors, True);
  }
}
