////////////////////////////////////////////////////////////////////////////
//
// dxfield.h - C++ wrapper around DX field object
//
////////////////////////////////////////////////////////////////////////////
#ifndef _DX_FIELD_H_
#define _DX_FIELD_H_

#include "dx++.h"
#include "dxiarray.h"
#include "hmap.h"

class DXField : public MyField {
  DeclareStandardMethodsNoPrint(DXField, MyField);
 public:
  // conversions back into DX Objects
  Field ToField() const;
  DXObject ToDXObject() const { return (DXObject)ToField(); }
  operator Field() const { return ToField(); }
  operator DXObject() const { return ToDXObject(); }

  // setting components
  void SetComponent(char*, DXObject, Boolean = False);
  void SetComponent(char*, Array, Boolean = False);

  // set component attributes
  void SetComponentAttribute(char*, char*, char*);
  void SetComponentAttribute(char*, char*, DXObject);

  // get components
  DXObject GetComponent(char*) const;
  DXArrayBase GetArrayComponent(char*) const;

  // creating new fields (see .c file for more info)
  DXField();
  DXField(const MyField&, Boolean = False, Boolean = False);
  DXField(const DXField&, Boolean = False, Boolean = False);
  DXField(int w, int h, int d = 1, Field = 0);
  DXField(Field f) { Init(f); }
  DXField(DXObject o) { Init(o); }

  // destroy a field
  ~DXField();
 protected:
  friend DXObject _DXExtractField(char**, DXObject, Type, Class, int, int, DXField*);
  // initialize DXField from real DX Object
  void Init(Field);
  void Init(DXObject o) { Init((Field)o); }

  // compute field dimensions
  void compute_field_size(int&, int&, int&);
  void set_field_colors();      // set DX Field colors from local copy
  void get_field_colors();      // copy DX Field colors into local copy
  void set_field_positions();   // set DX Field positions from local copy
  void get_field_positions();   // copy DX Field positions into local copy
 private:
  Boolean _own;   // is _field owned?
  Field _field;   // pointer to real DX Field object
};

#endif
