////////////////////////////////////////////////////////////////////////////
//
// dxiarray.h - templatized C++ wrapper around irregular Array objects
//
////////////////////////////////////////////////////////////////////////////
#ifndef _DXIARRAY_H_
#define _DXIARRAY_H_

#include "dxarray.h"
#include "point.h"

template <class T> class DXIrregularArray : public DXArray<T> {
 public:
  DXArrayBase* Copy() const
    {
      DXArray<T>* tmp = new DXIrregularArray<T>(Size(), Rank(), ShapeTable(), EltType(), EltCategory());
      T* tmpd = tmp->Data();
      T* d = Data();
      MemCopy(d, tmpd, EltSize()*Size());
      return tmp;
    }
  DXIrregularArray() : DXArray<T>() { }
  DXIrregularArray(Array a) : DXArray<T>(a) { }
  DXIrregularArray(const DXIrregularArray<T>& a) : DXArray<T>(a) { }
  DXIrregularArray(int size, int shape, Type t, Category c)
    : DXArray<T>(DXNewArray(t, c, 1, shape)) { SetSize(size); }
  DXIrregularArray(int size, int rank, int* shape, Type t, Category c)
    : DXArray<T>(DXNewArrayV(t, c, rank, shape)) { SetSize(size); }
};

// array of RGBColors
class ColorArray : public DXIrregularArray<RGBColor> {
 public:
  ColorArray() : DXIrregularArray<RGBColor>() { }
  ColorArray(Array a) : DXIrregularArray<RGBColor>(a) { }
  ColorArray(const ColorArray& a) : DXIrregularArray<RGBColor>(a) { }
  ColorArray(int size) : DXIrregularArray<RGBColor>(size, 3, TYPE_FLOAT, CATEGORY_REAL) { }
};

// array of positions (or other 3-vectors)
class PointArray : public DXIrregularArray<Point3d> {
 public:
  PointArray() : DXIrregularArray<Point3d>() { }
  PointArray(Array a) : DXIrregularArray<Point3d>(a) { }
  PointArray(const PointArray& a) : DXIrregularArray<Point3d>(a) { }
  PointArray(int size) : DXIrregularArray<Point3d>(size, 3, TYPE_FLOAT, CATEGORY_REAL) { }
};

// array of single float data (1-vectors)
class FloatArray : public DXIrregularArray<float> {
 public:
  FloatArray() : DXIrregularArray<float>() { }
  FloatArray(Array a) : DXIrregularArray<float>(a) { }
  FloatArray(const FloatArray& a) : DXIrregularArray<float>(a) { }
  FloatArray(int size) : DXIrregularArray<float>(size, 1, TYPE_FLOAT, CATEGORY_REAL) { }
};

#endif
