////////////////////////////////////////////////////////////////////////////
//
// dxmain.c - main for extensions when compiled as outboard modules
//
////////////////////////////////////////////////////////////////////////////
#include <assert.h>
#include <stream.h>
#include <strings.h>
#include "dx++.h"

typedef Error (*dx_fn)(DXObject*, DXObject*);

extern "C" {

// first declare all the new modules
#define dx_declare_module(x) extern Error m_##x(DXObject*, DXObject*);
#include "dxmodules.h"

// next build a table mapping from module names to functions
#undef dx_declare_module
#define dx_declare_module(x)  { #x, m_##x },

struct {
  char* name;
  dx_fn fn;
} dxmodules[] = {
#include "dxmodules.h"
  { 0, 0 }
};

// function pointer which is the correct outboard module to call
dx_fn dx_outboard_fn = 0;
int dxmain(int, char*[]);
};

int main(int argc, char* argv[])
{
  char* mname = rindex(argv[0], '/');
  int i;

  if(!mname)
    mname = argv[0];
  else
    mname++;

  // use argv[0] to determine which module we want to call
  for(i = 0; dxmodules[i].name; i++)
    if(!strcmp(dxmodules[i].name, mname))
      break;

  if(!dxmodules[i].name) {
    cout << "can't find module for " << mname << endl << flush;
    return -1;
  } else { // found correct module, so call regular dxmain
    cout << "calling module " << dxmodules[i].name << endl << flush;
    dx_outboard_fn = dxmodules[i].fn;
    return dxmain(argc, argv);
  }
}
