////////////////////////////////////////////////////////////////////////////
//
// rgbtogif.c - convert images saved by WriteImage in .rgb format to .gif
//
////////////////////////////////////////////////////////////////////////////
#include <fcntl.h>
#include "pnm++.h"

int main(int argc, char* argv[])
{
  const double fact = 1.0/255.0;
  static char buff[256];
  char* base = 0;
  char* src = 0;
  char* dest = 0;
  int width = 640;
  int height = 480;
  int read_size = 0;
  int i, j;

  for(i = 1; i < argc; i++) {
    if(!strcmp(argv[i], "-w"))
      width = atoi(argv[++i]);
    else if(!strcmp(argv[i], "-h"))
      height = atoi(argv[++i]);
    else if(!strcmp(argv[i], "-s"))
      read_size = 1;
    else if(!src)
      src = argv[i];
    else if(!dest)
      dest = argv[i];
  }

  char* tmp = rindex(src, '.');
  if(tmp) *tmp = 0;
  base = Strdup(src);
  if(tmp) *tmp = '.';

  if(!dest) dest = strcat(strcpy(new char[strlen(base)+5], base), ".gif");

  if(read_size) {
    char* sizef = strcat(strcpy(new char[strlen(base)+6], base), ".size");
    FILE* fs = fopen(sizef, "r");
    delete[] sizef;
    if(fs) {
      int crap;
      fscanf(fs, "%dx%dx%d", &width, &height, &crap);
      fclose(fs);
    } else {
      cout << "can't find .size file, exiting" << endl;
      return -1;
    }
  }

  cout << "src: " << src << endl;
  cout << "dest: " << dest << endl;
  cout << "width x height: " << width << " x " << height << endl;

  int fd = open(src, O_RDONLY);
  unsigned char* data = new unsigned char[3*width*height];
  read(fd, data, 3*width*height*sizeof(*data));
  close(fd);

  PercCounter pc("convert", width*height);
  PPM ppm(width, height);
  for(j = 0; j < height; j++) {
    for(i = 0; i < width; i++, pc++) {
      Real r = *data++ * fact;
      Real g = *data++ * fact;
      Real b = *data++ * fact;
      ppm(i, j) = RGB(r, g, b);
    }
  }
  ppm.Save(".tmp");

  system("ppmquant 256 .tmp > .tmp2");
  system("rm -f .tmp");
  sprintf(buff, "ppmtogif .tmp2 > %s", dest);
  system(buff);
  system("rm -f .tmp2");

  delete[] base;
  return 0;
}
