#include "hmap.h"

// craterize a height map.
void MyField::Craterize(int iter,         // number of craters to make
			Real depth_power, // factor controlling crater size
			Real depth_bias,  // another factor for crater size
			Real slope)       // controls slope of crater sides
{
  PercCounter pc("crater", iter);
  Real minh, maxh;
  int x, y;

  // scale heights so craterization will work right
  MinMaxHeights(minh, maxh);
  if(minh != maxh)
    ScaleHeights(65535.0);

  // set defaults
  if(depth_power == 0.0)
    depth_power = 1.5;
  if(depth_bias = 0.0)
    depth_bias = sqrt(0.5);

  for(int i = 0; i < iter; i++, pc++) {
    Real ihigh = sqrt(1.0 / (M_PI * (1.0 - rrand(1.0))));
//    Real ihigh = sqrt(1.0 / (M_PI * (1.0 - ngrand(1.0))));
    int cx = irand(Width()-2) + 1;
    int cy = irand(Height()-2) + 1;
    Real elev = 0.0;

    // small 1 pixel crater
    if(ihigh < 3.0) {

      // ignore most small craters
      if(0 && i % 1000 == 0) {
	// average heights over neighboring elements
	for(y = cy - 1; y <= cy + 1; y++)
	  for(x = cx - 1; x <= cx + 1; x++)
	    elev += Height(x, y);
	elev /= 9;

	// Perturb the mean elevation by a small random factor.
	Height(cx, cy) = elev + (ihigh >= 1.0) ? rrand(2.0) - 1.0 : 0.0;
      }

    } else {
      int cw = (int)max(2.0, ihigh / 3.0);
      int ch = (int)max(2.0, ihigh / 3.0);
      int xs = fixrng(cx - cw, 0, Width());
      int xe = fixrng(cx + cw, 0, Width());
      int ys = fixrng(cy - ch, 0, Height());
      int ye = fixrng(cy + ch, 0, Height());

      // find average height over impact area
      for(y = ys; y <= ye; y++)
	for(x = xs; x <= xe; x++)
	  elev += Height(x, y);
      elev /= (ye - ys) * (xe - xs);

      // interior of crater
      ihigh = max(2.0, ihigh);
      ch = cw = (int)ihigh;
      xs = fixrng(cx - cw, 0, Width());
      xe = fixrng(cx + cw, 0, Width());
      ys = fixrng(cy - ch, 0, Height());
      ye = fixrng(cy + ch, 0, Height());

      Real dp = pow(ihigh, depth_power);

      for(y = ys; y <= ye; y++) {
	Real dy = (cy - y) / (Real)ch;
	Real dy2 = dy * dy;

	for(x = xs; x <= xe; x++) {
	  Real& h = Height(x, y);
	  Real dx = (cx - x) / (Real)cw;
	  Real cd = (dx * dx) + dy2;
	  Real cd2 = cd * 2.25;
	  Real tcz = depth_bias - sqrt(fabs(1.0 - cd2));
	  Real cz = max((cd2 > 1.0) ? 0.0 : -10.0, tcz) * dp;
	  Real m = min(slope, cd);
	  Real ms = m * slope;
	  Real sp = (m - ms) / (slope - ms);

	  if(x == cx && y == cy && fabs(cz) < 1.0)
	    cz = cz < 0.0 ? -1.0 : 1.0;

	  Real newh = (elev + cz) * (1.0 - sp) + (h + cz) * sp;
	  h = max(1000.0, min(64000.0, newh));
	}
      }
    }
  }

#if 0
  // scale back to original
  if(minh != maxh) {
    ScaleHeights(maxh - minh);
    *this += minh;
  }
#endif
}

#if 0

/* craters are too deep!!, need to make new raise fn */

/* could use this for volcanoes too! */
/* this is ok but there is probably a nice "crater"
 * function that could be used instead */
void MyField::Craterize(int xc, int yc, int crad, Real chigh, int irad, Real ihigh)
{
#if 1
  Sraise(xc, yc, irad, -ihigh, 0.0 /*ihigh * 0.2*/, 1.0);
#endif
#if 0
  Craise(xc, yc, crad, chigh, 0.0, 0.3);
#endif
}
#endif
