#include "texture.h"
#include "matrix3d.h"
#include "array.h"
#include "pnm++.h"
#include "conv.h"
#include "palette.h"
#include "hmap.h"

/* raise/lower pyramidal area */
/* first tier has 0 hieght - wrong? */
void MyField::Praise(int xc,       // x center
		     int yc,       // y center
		     int hwidth,   // half width
		     int hheight,  // half height
		     Real amt,     // amount to raise/lower
		     Real zoff,    // z offset for pyramid base
		     Real truncp)  // can truncate pyramid to arbitrary height
{
  int zmax = (int)round(fabs(amt / Granularity()));
  int trunc = (int)(zmax * truncp);
  int i, j, k;

  // reject 0 height pyramids
  if(zmax == 0.0)
    return;

  for(i = 0; i <= trunc; i++) {
    Real perc = (Real)(zmax - i) / (Real)zmax;
    Real curr_raise = amt * (1.0 - perc) + zoff;
    int curr_width = (int)(hwidth * perc);
    int curr_height = (int)(hheight * perc);
    int xstart = fixrng(xc - curr_width, 0, Width());
    int xend = fixrng(xc + curr_width, 0, Width());
    int ystart = fixrng(yc - curr_height, 0, Height());
    int yend = fixrng(yc + curr_height, 0, Height());

    if(i < trunc) {
      Real next_perc = (Real)(zmax - i - 1) / (Real)zmax;
      int next_width = (int)(hwidth * next_perc);
      int next_height = (int)(hheight * next_perc);
      int wdiff = curr_width - next_width;
      int hdiff = curr_height - next_height;

      if(!wdiff && !hdiff) {
	/* do nothing */
      } else if(0 && hdiff == 1 && !wdiff) {
	for(j = xstart; j <= xend; j++) {
	  (*this)(j, ystart).z += curr_raise;
	  (*this)(j, yend).z += curr_raise;
	}
      } else if(0 && wdiff == 1 && !hdiff) {
	for(j = ystart; j <= yend; j++) {
	  (*this)(xstart, j).z += curr_raise;
	  (*this)(xend, j).z += curr_raise;
	}
      } else if(0 && wdiff == 1 && hdiff == 1) {
	for(j = xstart; j <= xend; j++) {
	  (*this)(j, ystart).z += curr_raise;
	  (*this)(j, yend).z += curr_raise;
	}
	for(j = ystart+1; j < yend; j++) {
	  (*this)(xstart, j).z += curr_raise;
	  (*this)(xend, j).z += curr_raise;
	}
      } else {
	int next_xstart = fixrng(xc - next_width, 0, Width());
	int next_xend = fixrng(xc + next_width, 0, Width());
	int next_ystart = fixrng(yc - next_height, 0, Height());
	int next_yend = fixrng(yc + next_height, 0, Height());
#if 1
	Point3d* rptr1 = &(*this)(xstart, ystart);
	Point3d* rptr2 = &(*this)(next_xend+1, ystart);
	int inc1 = Width() - (next_xstart - xstart);
	int inc2 = Width() - (xend - next_xend);

	for(k = yend - ystart; k >= 0; k--) {
	  for(j = next_xstart - xstart - 1; j >= 0; j--, rptr1++)
	    rptr1->z += curr_raise;
	  for(j = xend - next_xend - 1; j >= 0; j--, rptr2++)
	    rptr2->z += curr_raise;
	  rptr1 += inc1;
	  rptr2 += inc2;
	}
	rptr1 = &(*this)(next_xstart, ystart);
	rptr2 = &(*this)(next_xstart, next_yend+1);
	inc1 = Width() - (next_xend - xstart);

	for(k = next_ystart - ystart - 1; k >= 0; k--) {
	  for(j = next_xend - next_xstart; j >= 0; j--, rptr1++)
	    rptr1->z += curr_raise;
	  rptr1 += inc1;
	}
	for(k = yend - next_yend - 1; k >= 0; k--) {
	  for(j = next_xend - next_xstart; j >= 0; j--, rptr2++)
	    rptr2->z += curr_raise;
	  rptr2 += inc1;
	}
#else
	for(k = ystart; k <= yend; k++) {
	  for(j = xstart; j < next_xstart; j++)
	    (*this)(j, k).z += curr_raise;
	  for(j = next_xend+1; j <= xend; j++)
	    (*this)(j, k).z += curr_raise;
	}
	for(j = next_xstart; j <= next_xend; j++) {
	  for(k = ystart; k < next_ystart; k++)
	    (*this)(j, k).z += curr_raise;
	  for(k = next_yend+1; k <= yend; k++)
	    (*this)(j, k).z += curr_raise;
	}
#endif
      }
    } else {
      Point3d* rptr = &(*this)(xstart, ystart);
      int inc = Width() - (xend - xstart) - 1;
      for(k = ystart; k <= yend; k++) {
	for(j = xend - xstart; j >= 0; j--, rptr++)
	  rptr->z += curr_raise;
	rptr += inc;
      }
    }
  }
}

/* a_raise() - fn(zmax, curr_z, xc, yc, other), doesn't need to use
   all these args to compute fn, must return ymin, ystart, ysize &
   fill left/right arrays
*/

// raise/lower a cone shaped area in the field
void MyField::Craise(int xc,         // x center
		     int yc,         // y center
		     int rad,        // cone radius
		     Real amt,       // amount to raise/lower
		     Real zoff,      // z offset of cone base
		     Real truncp)    // truncate cone to arbitrary height
{
  Real* left = new Real[2*rad+1];    // cone shape arrays
  Real* right = new Real[2*rad+1];
  Real* lleft = new Real[2*rad+1];
  Real* lright = new Real[2*rad+1];
  int zmax = (int)(round(fabs(amt / Granularity())));
  int trunc = (int)(zmax * truncp);
  int lystart = 0, lyend = 0;
  int z;

  // reject 0 height cones
  if(zmax == 0.0)
    return;

  for(z = trunc; z >= 0; z--) {
    Real perc = (Real)(zmax - z) / (Real)zmax;
    Real curr_raise = amt * (1.0 - perc) + zoff;
    int ystart, yend, ymin, yindex, ysize;

    /* compute function here and fill left/right arrays, set ymin & ymax */
    int radius = (int)(rad * perc);
    int x = 0;
    int y = radius;
    int d = 1 - radius;

    ymin = yc - radius;     /* min value of function at top */
    yindex = rad - radius;  /* start index into left & right arrays */
    ysize = 2*radius;       /* size of values in left & right arrays */

    /* top/middle/bottom */
    if(radius != 0) {
      left[rad-radius] = 0;
      right[rad-radius] = 0;
      left[rad] = -radius;
      right[rad] = radius;
      left[rad+radius] = 0;
      right[rad+radius] = 0;
    } else {
      left[rad] = 0;
      right[rad] = 0;
    }

    while(y > x) {
      if(d < 0) {
	d += 2*x + 3;
	x++;
      } else {
	d += 2*(x-y)+5;
	x++;
	y--;
      }
      left[rad - y] = -x;
      right[rad - y] = x;
      left[rad + y] = -x;
      right[rad + y] = x;
      left[rad - x] = -y;
      right[rad - x] = y;
      left[rad + x] = -y;
      right[rad + x] = y;
    }

    if(ymin < 0) yindex -= ymin;
    ystart = fixrng(ymin, 0, Height());
    yend = fixrng(ymin + ysize, 0, Height());

    if(z == trunc) {
      for(y = ystart; y <= yend; y++, yindex++) {
	int lr = fixrng((int)(xc + left[yindex]), 0, Width());
	int rr = fixrng((int)(xc + right[yindex]), 0, Width());
	for(x = lr; x <= rr; x++)
	  (*this)(x, y).z += curr_raise;
      }
    } else {
      for(y = ystart; y < lystart; y++, yindex++) {
	int lr = fixrng((int)(xc + left[yindex]), 0, Width());
	int rr = fixrng((int)(xc + right[yindex]), 0, Width());
	for(x = lr; x <= rr; x++)
	  (*this)(x, y).z += curr_raise;
      }

      for(y = lystart; y <= lyend; y++, yindex++) {
	int lr = fixrng((int)(xc + left[yindex]), 0, Width());
	int rr = fixrng((int)(xc + right[yindex]), 0, Width());
	int llr = fixrng((int)(xc + lleft[yindex]), 0, Width());
	int lrr = fixrng((int)(xc + lright[yindex]), 0, Width());
	for(x = lr; x < llr; x++)
	  (*this)(x, y).z += curr_raise;
	for(x = lrr+1; x <= rr; x++)
	  (*this)(x, y).z += curr_raise;
      }

      for(y = lyend+1; y <= yend; y++, yindex++) {
	int lr = fixrng((int)(xc + left[yindex]), 0, Width());
	int rr = fixrng((int)(xc + right[yindex]), 0, Width());
	for(x = lr; x <= rr; x++)
	  (*this)(x, y).z += curr_raise;
      }
    }

    lystart = ystart;
    lyend = yend;
    MemCopy(left, lleft, 2*rad*sizeof(*left));
    MemCopy(right, lright, 2*rad*sizeof(*right));
  }

  delete[] left;
  delete[] right;
  delete[] lleft;
  delete[] lright;
}


// raise/lower a hemisphere shaped area in the field
void MyField::Sraise(int xc,         // x center
		     int yc,         // y center
		     int rad,        // hemisphere radius
		     Real amt,       // amount to raise/lower
		     Real zoff,      // z offset of hemisphere base
		     Real truncp)    // truncate hemisphere to arbitrary height
{
  Real* left = new Real[2*rad+1];
  Real* right = new Real[2*rad+1];
  Real* lleft = new Real[2*rad+1];
  Real* lright = new Real[2*rad+1];
  int zmax = (int)round(fabs(amt / Granularity()));
  int trunc = (int)(zmax * truncp);
  int lystart = 0, lyend = 0;
  int z;

  // reject 0 height hemispheres
  if(zmax == 0.0)
    return;

  for(z = trunc; z >= 0; z--) {
    Real perc = (Real)(zmax - z) / (Real)zmax;
    Real curr_raise = amt * (1.0 - perc) + zoff;
    int ystart, yend, ymin, yindex, ysize;

    /* compute function here and fill left/right arrays, set ymin & ymax */
    int radius = (int)(rad * fsin(acos(1.0 - perc)));
    int x = 0;
    int y = radius;
    int d = 1 - radius;

    ymin = yc - radius;     /* min value of function at top */
    yindex = rad - radius;  /* start index into left & right arrays */
    ysize = 2*radius;       /* size of values in left & right arrays */

    /* top/middle/bottom */
    if(radius != 0) {
      left[rad-radius] = 0;
      right[rad-radius] = 0;
      left[rad] = -radius;
      right[rad] = radius;
      left[rad+radius] = 0;
      right[rad+radius] = 0;
    } else {
      left[rad] = 0;
      right[rad] = 0;
    }

    while(y > x) {
      if(d < 0) {
	d += 2*x + 3;
	x++;
      } else {
	d += 2*(x-y)+5;
	x++;
	y--;
      }
      left[rad - y] = -x;
      right[rad - y] = x;
      left[rad + y] = -x;
      right[rad + y] = x;
      left[rad - x] = -y;
      right[rad - x] = y;
      left[rad + x] = -y;
      right[rad + x] = y;
    }

    if(ymin < 0) yindex -= ymin;
    ystart = fixrng(ymin, 0, Height());
    yend = fixrng(ymin + ysize, 0, Height());

    if(z == trunc) {
      for(y = ystart; y <= yend; y++, yindex++) {
	int lr = fixrng((int)(xc + left[yindex]), 0, Width());
	int rr = fixrng((int)(xc + right[yindex]), 0, Width());
	for(x = lr; x <= rr; x++)
	  (*this)(x, y).z += curr_raise;
      }
    } else {
      for(y = ystart; y < lystart; y++, yindex++) {
	int lr = fixrng((int)(xc + left[yindex]), 0, Width());
	int rr = fixrng((int)(xc + right[yindex]), 0, Width());
	for(x = lr; x <= rr; x++)
	  (*this)(x, y).z += curr_raise;
      }

      for(y = lystart; y <= lyend; y++, yindex++) {
	int lr = fixrng((int)(xc + left[yindex]), 0, Width());
	int rr = fixrng((int)(xc + right[yindex]), 0, Width());
	int llr = fixrng((int)(xc + lleft[yindex]), 0, Width());
	int lrr = fixrng((int)(xc + lright[yindex]), 0, Width());
	for(x = lr; x < llr; x++)
	  (*this)(x, y).z += curr_raise;
	for(x = lrr+1; x <= rr; x++)
	  (*this)(x, y).z += curr_raise;
      }

      for(y = lyend+1; y <= yend; y++, yindex++) {
	int lr = fixrng((int)(xc + left[yindex]), 0, Width());
	int rr = fixrng((int)(xc + right[yindex]), 0, Width());
	for(x = lr; x <= rr; x++)
	  (*this)(x, y).z += curr_raise;
      }
    }

    lystart = ystart;
    lyend = yend;
    MemCopy(left, lleft, 2*rad*sizeof(*left));
    MemCopy(right, lright, 2*rad*sizeof(*right));
  }

  delete[] left;
  delete[] right;
  delete[] lleft;
  delete[] lright;
}


/* this procedure not truly fractalish because scale doesn't change
 * as iterations proceed.  scale should get smaller and smaller as
 * a function of H (eg. pow(0.5, 0.5 * H)) at each iteration.  Also
 * number of raises should increase exponentially(?) for each iteration
 */
void MyField::Raise(int iter, char* kind)
{
  int raisek = 0;

  if(!strcmp(kind, "sphere")) {
    raisek = 0;
  } else if(!strcmp(kind, "pyramid")) {
    raisek = 1;
  } else if(!strcmp(kind, "cone")) {
    raisek = 2;
  } else {
    return;
  }

  PercCounter pc("raise", iter);
  for(int i = 0; i < iter; i++, pc++) {
    int xc = irand(Width());
    int yc = irand(Height());
    int hw = irand(Width()/3) + 5;
    int hh = irand(Height()/3) + 5;
//    int hw = (int)ngrand(Width()/3) + 5;
//    int hh = (int)ngrand(Height()/3) + 5;
    Real range = 0.0, amt;

    if(raisek == 1) {
      if(hw < hh/2) hw = hh/2;
      if(hh < hw/2) hh = hw/2;
    }

    if(raisek == 0 || raisek == 2)
      range = 4.0 * hw;
    else if(raisek == 1)
      range = 2.0 * (hw + hh);

    amt = rrand(2.0*range) - range;
    //assert(amt >= -range && amt <= range);

    if(raisek == 0)
      Sraise(xc, yc, hw, amt, 0.0, 1.0);
    else if(raisek == 1)
      Praise(xc, yc, hw, hh, amt, 0.0, 1.0);
    else if(raisek == 2)
      Craise(xc, yc, hw, amt, 0.0, 1.0);
  }
  ScaleHeights((Height() + Width())/10.0);
}


// print small amount of info about a field
ostream& MyField::Print(ostream& os, int v) const
{
  Object::Print(os);
  os << "(" << Width() << " x " << Height() << " x " << Depth() << ", ";
  if(color()) {
    if(color(0,0).IsA(RGB::Id()))
      os << "Color, ";
    else
      os << "GrayScale, ";
  } else {
    os << "NoColor, ";
  }
  os << (Own() ? "Own" : "NoOwn") << ")";
  if(v) {
    Real min, max, ave = AverageHeight();
    MinMaxHeights(min, max);
    os << "(" << min << ", " << max << ", " << ave << ")";
  }
  return os;
}

// copy data from f
void MyField::CopyData(const MyField& f)
{
  MemCopy(f._data, _data, Size()*sizeof(*_data));
}

// copy colors from f
void MyField::CopyColors(const MyField& f)
{
  MemCopy(f._color, _color, Size()*sizeof(*_color));
}

// initialize grid coordinates in regular manner using
// given origin and deltas
void MyField::InitRegularGrid(Real ox, Real oy, Real oz,
			      Real dx, Real dy, Real dz)
{
  Real x = ox, y = oy, z = oz;
  for(int k = 0; k < Depth(); k++, z += dz, y = oy)
    for(int j = 0; j < Height(); j++, y += dy, x = ox)
      for(int i = 0; i < Width(); i++, x += dx)
	(*this)(i, j, k) = Point3d(x, y, z);
}

// rotate field through 3 dimensions
void MyField::Rotate(Real xa, Real ya, Real za)
{
  MyField newf((int)(Width() * 1.5), (int)(Height() * 1.5), 1, Granularity());
  Real cx = Width()/2.0;
  Real cy = Height()/2.0;
  Matrix3d rot = RotateX3d(xa) * RotateY3d(ya) * RotateZ3d(za);

  cout << "rotating" << endl;
  for(int x = 0; x < Width(); x++) {
    for(int y = 0; y < Height(); y++) {
      Vector3d vec = rot * Vector3d(x - cx, y - cy, (*this)(x, y).z);
      newf((int)vec.i, (int)vec.j) = (*this)(x, y);
    }
  }
  cout << "done rotating" << endl;

  newf._own = False;
  set_size(newf._width, newf._height, newf._depth, newf._gran);
  set_data(newf._data);
  set_colors(0);
  _own = True;
}

// apply a texture to this field
void MyField::Texturize(const Texture& t)
{
  PercCounter pc("texturize", Size());
  Point3d* ptr = data();
  Real dx = 1.0/Width();
  Real dy = 1.0/Height();
  Real dz = 1.0/Depth();

  for(int k = 0; k < Depth(); k++) {
    for(int j = 0; j < Height(); j++) {
      for(int i = 0; i < Width(); i++, pc++) {
	Point3d& pp = *ptr++;
	pp.z = t(pp.x * dx, pp.y * dy, pp.z * dz);
      }
    }
  }
}

// colorize field by height with the given palette.
// minp and maxp are used to select a range of palette values
void MyField::Colorize(const Palette& p, Real minp, Real maxp)
{
  if(!color()) set_colors(new RGB[Size()]);
  Color* cptr = color();
  Point3d* dptr = data();
  Real min, max;

  MinMaxHeights(min, max);
  for(int i = Size() - 1; i >= 0; i--, dptr++) {
    Real val = (((dptr->z - min) * (maxp - minp)) / (max - min)) + minp;
    int index = (int)(val * (p.Size() - 2) + 1);
    *cptr++ = p[index];
  }
}

// gray scale a field by height using gsmin and
// gsmax to select a range of intensity values
void MyField::IntensityMap(GrayScale gsmin, GrayScale gsmax)
{
  Real min, max;
  Real gmin = gsmin.Intensity();
  Real gmax = gsmax.Intensity();
  Point3d* dptr = data();

  if(!color()) set_colors(new GrayScale[Size()]);
  GrayScale* cptr = (GrayScale*)color();

  MinMaxHeights(min, max);
  for(int i = Size() - 1; i >= 0; i--, dptr++) {
    Real val = (dptr->z - min) / (max - min);
    *cptr++ = (gmax - gmin) * val + gmin;
  }
}

// remove field colors
void MyField::DeColorize()
{
  set_colors(0);
}

// save a field to a PNM
int MyField::Save(char* file, int k) const
{
  PNM* pnm = ToPNM(k);
  pnm->Save(file);
  delete pnm;
  return 1;
}

// save a field to a gif
int MyField::SaveGif(char* file, Boolean quant) const
{
  static char command[256];
  int res = Save("/tmp/tmp.ppm");
  if(!res) return res;
  if(quant) {
    system("ppmquant 256 /tmp/tmp.ppm > /tmp/tmp2.ppm");
    system("mv /tmp/tmp2.ppm /tmp/tmp.ppm");
  }
  sprintf(command, "ppmtogif /tmp/tmp.ppm > %s; rm /tmp/tmp.ppm", file);
  system(command);
  return res;
}

// load a field from a PNM using p to peform the color to
// height mapping.  If p is not given then the color
// intensity is used to compute the height
int MyField::Load(char* file, const Palette* p)
{
  PNM* pnm = PNM::Load(file);
  set_data(0);
  set_colors(0);
  *this = MyField(*pnm, p);
  return 1;
}

// add two fields positions
const MyField& MyField::operator+=(const MyField& f)
{
  for(int i = 0; i < Size(); i++)
    (*this)(i) += f(i);
  return *this;
}

// subtract two fields positions
const MyField& MyField::operator-=(const MyField& f)
{
  for(int i = 0; i < Size(); i++)
    (*this)(i) -= f(i);
  return *this;
}

// multiply two fields positions
const MyField& MyField::operator*=(const MyField& f)
{
  for(int i = 0; i < Size(); i++)
    (*this)(i) *= f(i);
  return *this;
}

// divide two fields positions
const MyField& MyField::operator/=(const MyField& f)
{
  for(int i = 0; i < Size(); i++)
    (*this)(i) /= f(i);
  return *this;
}


// add a point to this field's positions
const MyField& MyField::operator+=(const Point3d& p)
{
  for(int i = 0; i < Size(); i++)
    (*this)(i) += p;
  return *this;
}

// subtract a point to this field's positions
const MyField& MyField::operator-=(const Point3d& p)
{
  for(int i = 0; i < Size(); i++)
    (*this)(i) -= p;
  return *this;
}

// multiply this field's positions by a point
const MyField& MyField::operator*=(const Point3d& p)
{
  for(int i = 0; i < Size(); i++)
    (*this)(i) *= p;
  return *this;
}

// divide this field's positions by a point
const MyField& MyField::operator/=(const Point3d& p)
{
  for(int i = 0; i < Size(); i++)
    (*this)(i) /= p;
  return *this;
}

// add a constant value to this field's heights
const MyField& MyField::operator+=(Real v)
{
  for(int i = 0; i < Size(); i++)
    Height(i) += v;
  return *this;
}

// subtract a constant value from this field's heights
const MyField& MyField::operator-=(Real v)
{
  for(int i = 0; i < Size(); i++)
    Height(i) -= v;
  return *this;
}

// multiply this field's heights by a constant value
const MyField& MyField::operator*=(Real v)
{
  for(int i = 0; i < Size(); i++)
    Height(i) *= v;
  return *this;
}

// divide this field's heights by a constant value
const MyField& MyField::operator/=(Real v)
{
  for(int i = 0; i < Size(); i++)
    Height(i) /= v;
  return *this;
}

// merge two fields in an arbitrary way
void MyField::Merge(int xa,             // merge start x
		    int ya,             // merge start y
		    int w,              // width of merge area
		    int h,              // height of merge area
		    const MyField& f,   // other field
		    int xb,             // other field merge start x
		    int yb,             // other field merge start y
		    int xo,             // destination x
		    int yo,             // destination y
		    FieldMergeFn fn,    // merge function
		    void* env)          // env parameter for merge fn
{
  for(int i = 0; i < w; i++)
    for(int j = 0; j < h; j++)
      (*this)(xo+i, yo+j) = fn((*this)(xa+i, ya+j), f(xb+i, yb+j), env);
}

// average heights of two fields
void MyField::AverageHeights(const MyField& f)
{
  for(int i = 0; i < Size(); i++)
    Height(i) = (Height(i) + f.Height(i)) / 2.0;
}

// add heights of two fields
void MyField::AddHeights(const MyField& f)
{
  for(int i = 0; i < Size(); i++)
    Height(i) += f.Height(i);
}

// subtract heights of two fields
void MyField::SubtractHeights(const MyField& f)
{
  for(int i = 0; i < Size(); i++)
    Height(i) -= f.Height(i);
}

// return an array of the heights of this field.
// the array must be freed by the user when done.
Real* MyField::Heights() const
{
  Real* h = new Real[Size()];
  for(int i = 0; i < Size(); i++)
    h[i] = Height(i);
  return h;
}

// set the heights of this field from the array h
void MyField::SetHeights(Real* h)
{
  for(int i = 0; i < Size(); i++)
    Height(i) = h[i];
}

// set all the heights in this field to h
void MyField::SetHeight(Real h)
{
  for(int i = 0; i < Size(); i++)
    Height(i) = h;
}

// set the heights of rectangular area to h
void MyField::SetHeight(int x1, int y1, int x2, int y2, Real h)
{
  for(int i = x1; i < x2; i++)
    for(int j = y1; j < y2; j++)
      Height(i, j) = h;
}

// find the minimum and maximum heights of this field
void MyField::MinMaxHeights(Real& min, Real& max) const
{
  MinMaxDim(2, min, max);
}

// find the minimum and maximum heights of this field
void MyField::MinMaxDim(int dim, Real& min, Real& max) const
{
  Point3d pmin, pmax;
  MinMaxDim(dim, pmin, pmax);
  min = pmin[dim];
  max = pmax[dim];
}

// find the minimum and maximum points of this field
void MyField::MinMaxDim(int dim, Point3d& minp, Point3d& maxp) const
{
  Real min = 0.0, max = 0.0;
  for(int i = 0; i < Size(); i++) {
    Point3d p = (*this)(i);
    if(!i || min > p[dim]) { min = p[dim]; minp = p; }
    if(!i || max < p[dim]) { max = p[dim]; maxp = p; }
  }
}

// scale the positions of a field
void MyField::ScalePositions(Real xs, Real ys, Real zs)
{
  Real xmin, xmax, ymin, ymax, zmin, zmax;

  for(int i = 0; i < Size(); i++) {
    Point3d p = (*this)(i);
    if(!i) {
      xmin = xmax = p.x;
      ymin = ymax = p.y;
      zmin = zmax = p.z;
    } else {
      if(xmin > p.x) xmin = p.x;
      if(xmax < p.x) xmax = p.x;
      if(ymin > p.y) ymin = p.y;
      if(ymax < p.y) ymax = p.y;
      if(zmin > p.z) zmin = p.z;
      if(zmax < p.z) zmax = p.z;
    }
  }

  for(i = 0; i < Size(); i++) {
    Point3d& p = (*this)(i);
    if(xs != 0.0) p.x = xs * (p.x - xmin) / (xmax - xmin);
    if(ys != 0.0) p.y = ys * (p.y - ymin) / (ymax - ymin);
    if(zs != 0.0) p.z = zs * (p.z - zmin) / (zmax - zmin);
  }
}

// scale the heights of this field (makes heights fall between 0 and scale)
void MyField::ScaleHeights(Real scale)
{
  Real min, max;
  MinMaxHeights(min, max);
  for(int i = 0; i < Size(); i++)
    Height(i) = scale * ((Height(i) - min) / (max - min));
}

// compute the average height of this field
Real MyField::AverageHeight() const
{
  Real sum = 0.0;
  for(int i = 0; i < Size(); i++)
    sum += Height(i);
  return sum / Size();
}

// compute the average height over a region
Real MyField::AverageHeight(int x1, int y1, int x2, int y2) const
{
  Real sum = 0.0;
  for(int i = x1; i < x2; i++)
    for(int j = y1; j < y2; j++)
      sum += Height(i, j);
  return sum / ((x2 - x1) * (y2 - y1));
}

// smooth heights of this field using gaussian filter
void MyField::Smooth(Real sigma)
{
  Real* d = Heights();
  ::Smooth(d, Width(), Height(), sigma);
  SetHeights(d);
  delete[] d;
}

// perform edge detection on this field
void MyField::EdgeDetect(Real thresh)
{
  unsigned char* edge = new unsigned char[Size()];
  Real* hdata = Heights();
  ::EdgeDetect(hdata, Width(), Height(), thresh, edge);
  for(int i = 0; i < Size(); i++)
    Height(i) = edge[i];
  delete[] hdata;
  delete[] edge;
}

// perform arbitrary convolution on this field's heights
void MyField::Convolute(const Filter& f)
{
  Real* d = Heights();
  Real* newd = new Real[Size()];
  ::Convolute(d, Width(), Height(), (Filter&)f, newd);
  SetHeights(newd);
  delete[] d;
  delete[] newd;
}

// convert this field to a PNM
// (PPM if field has been colored or PGM if field has been gray scaled)
PNM* MyField::ToPNM(int k) const
{
  PNM* pnm = 0;

  if(_color) {
    if(color(0,0).IsA(RGB::Id()))
      pnm = new PPM(Width(), Height());
    else
      pnm = new PGM(Width(), Height());
    for(int i = 0; i < Width(); i++)
      for(int j = 0; j < Height(); j++)
	(*pnm)(i, j) = color(i, j, k);
  }
  return pnm;
}

// set the size components of this field
// deletes data and colors components since they probably
// will no longer be correct
void MyField::set_size(int w, int h, int d, Real gran)
{
  if(w) _width = w;
  if(h) _height = h;
  if(d) _depth = d;
  if(gran) _gran = gran;
  set_data(0);
  set_colors(0);
}

// set the data component of this field
// deletes the old component if it exists and it was owned
void MyField::set_data(Point3d* p)
{
  if(_own && _data && p != _data)
    delete[] _data;
  _data = p;
}

// set the color component of this field
// deletes the old component if it exists and it was owned
void MyField::set_colors(Color* c)
{
  if(_own && _color && c != _color)
    delete[] _color;
  _color = c;
}


// create empty field
MyField::MyField()
: _width(0), _height(0), _depth(0), _gran(1.0), _data(0),
  _color(0), _own(False)
{
}

// create empty field
MyField::MyField(const MyField& a, const MyField& b)
: _width(a.Width()+b.Width()), _height(max(a.Height(), b.Height())),
  _depth(1), _gran(1.0), _data(0), _color(0),
  _own(True)
{
  int i, j;

  set_data(new Point3d[Size()]);
  InitRegularGrid(0.0, 0.0, 0.0, 1.0, 1.0, 0.0);

  for(i = 0; i < a.Width(); i++)
    for(j = 0; j < a.Height(); j++)
      (*this)(i, j) = a(i, j);

  for(i = a.Width(); i < Width(); i++)
    for(j = 0; j < b.Height(); j++)
      (*this)(i, j) = b(i-a.Width(), j);
}

// load field from file (see Load())
MyField::MyField(char* file, const Palette* p)
: _gran(1.0), _data(0), _color(0), _own(False)
{
  Load(file, p);
}

// convert PNM to field using given palette for height mapping
MyField::MyField(const PNM& pnm, const Palette* p)
: _width(pnm.Width()), _height(pnm.Height()), _depth(1),
  _gran(1.0), _data(new Point3d[pnm.Size()]), _color(0),
  _own(True)
{
  PNM& vpnm = (PNM&)pnm;

  InitRegularGrid(0.0, 0.0, 0.0, 1.0, 1.0, 0.0);

  if(pnm.IsA(PPM::Id())) {
    _color = new RGB[pnm.Size()];
    for(int i = 0; i < Width(); i++) {
      for(int j = 0; j < Height(); j++) {
	RGB rgb = (RGB)vpnm(i, j);
	if(p)
	  (*this)(i, j).z = ((*p)[rgb]) / ((Real)p->Size());
	else
	  (*this)(i, j).z = rgb.Intensity();
	color(i, j) = rgb;
      }
    }
  } else {
    _color = new GrayScale[pnm.Size()];
    for(int i = 0; i < Width(); i++) {
      for(int j = 0; j < Height(); j++) {
	GrayScale g = (GrayScale)vpnm(i, j);
	(*this)(i, j).z = g.Intensity();
	color(i, j) = g;
      }
    }
  }
}

// copy or reference field f to create a new field
MyField::MyField(const MyField& f, Boolean copy, Boolean own)
: _width(f.Width()), _height(f.Height()), _depth(f.Depth()),
  _gran(f.Granularity()), _data(0), _color(0), _own(own)
{
  if(copy)
    _data = (Point3d*)MemCopy(f.data(), new Point3d[Size()], Size()*sizeof(*_data));
  else
    _data = f.data();

  if(copy && f.color()) {
    if(f.color(0).IsA(RGB::Id()))
      _color = new RGB[Size()];
    else
      _color = new GrayScale[Size()];
    MemCopy(f.color(), _color, Size()*sizeof(*_color));
  } else {
    _color = f.color();
  }
}


// create new empty field of the given size
MyField::MyField(int w, int h, int d, Real g)
: _width(w), _height(h), _depth(d), _gran(g),
  _data(new Point3d[w*h*d]), _color(0), _own(True)
{
  assert(w && h && d && g);
  InitRegularGrid(0.0, 0.0, 0.0, 1.0, 1.0, 0.0);
}

// destroy a field (and any owned components)
MyField::~MyField()
{
  if(_own) {
    delete[] _data;
    delete[] _color;
  }
}
