////////////////////////////////////////////////////////////////////////////
//
// array.h - templatized array operations (min/max, scale and profile)
//
////////////////////////////////////////////////////////////////////////////
#ifndef _ARRAY_OP_H_
#define _ARRAY_OP_H_

// find min and max values of given array
template <class T>
inline void MinMaxArray(T* in, int num, T& min, T& max)
{
  for(int i = num - 1; i >= 0; i--) {
    const T& pd = *in++;
    if(i == num - 1) {
      min = max = pd;
    } else {
      if(pd > max) max = pd;
      if(pd < min) min = pd;
    }
  }
}

// scale array using sf as the scale function.
// if sf is not given then the array is normalized
template <class T>
inline void ScaleArray(T* in, int num, T* out, T (*sf)(T&, T&, void*), void* env)
{
  T* rptr = in;
  T* nptr = out;
  T maxr, minr;

  MinMaxArray(in, num, minr, maxr);
  for(int i = num - 1; i >= 0; i--, nptr++) {
    T tmp = (*rptr++ - minr) / (maxr - minr);
    *nptr = sf ? sf(tmp, *nptr, env) : tmp;
  }
}

// create a "profile" of an array
template <class T>
inline void ProfileArray(T* in, int width, T* out, Real scale, T val)
{
  int height = (int)(width / scale);
  Real hmin, hmax;

  MinMaxArray<T>(in, width, hmin, hmax);
  for(int x = 0; x < width; x++) {
    Real perc = (in[x] - hmin) / (hmax - hmin);
    int curr_max = (int)(((Real)width * perc) / (2.0 * scale));
    for(int y = 0; y <= curr_max; y++)
      out[x + (height - y)*width] = val;
  }
}

#endif
