////////////////////////////////////////////////////////////////////////////
//
// complex.h - simple complex number class
//
////////////////////////////////////////////////////////////////////////////
#ifndef _COMPLEX_H_
#define _COMPLEX_H_

#include "real.h"
#include "trig.h"

// complex number
class MyComplex {
 public:
  Real r, i;  // real and imaginary components

  // multiply two complex numbers
  MyComplex operator*(const MyComplex& c) const
    { return MyComplex(r * c.r - i * c.i, r * c.i + i * c.r); }

  // divide two complex numbers
  // (a + bi)/(c + di) = (ac + bd)/(c^2 + d^2) + (bc - ad)i/(c^2 + d^2)
  MyComplex operator/(const MyComplex& c) const
    {
      Real div = c.r * c.r + c.i * c.i;
      return MyComplex((r * c.r + i * c.i) / div, (i * c.r - r * c.i) / div);
    }

  // add two complex numbers
  MyComplex operator+(const MyComplex& c) const
    { return MyComplex(r + c.r, i + c.i); }

  // divide two complex numbers
  MyComplex operator-(const MyComplex& c) const
    { return MyComplex(r - c.r, i - c.i); }

  // assignment operator versions of above functions
  const MyComplex& operator+=(const MyComplex& c) { return (*this = *this + c); }
  const MyComplex& operator-=(const MyComplex& c) { return (*this = *this - c); }
  const MyComplex& operator*=(const MyComplex& c) { return (*this = *this * c); }
  const MyComplex& operator/=(const MyComplex& c) { return (*this = *this / c); }

  // perform exponentiation of complex number
  MyComplex operator^(Real phi) const;

  // create empty complex number
  MyComplex() : r(0.0), i(0.0) { }

  // copy another complex number
  MyComplex(const MyComplex& c) : r(c.r), i(c.i) { }

  // create complex number from real and imaginary components
  MyComplex(Real rr, Real ii) : r(rr), i(ii) { }

  // destroy complex number
  ~MyComplex() { }

  // print complex number
  friend ostream& operator<<(ostream&, const MyComplex&);
  ostream& Print(ostream& os, int=0) const;
};

#endif
