#include <stream.h>
#include <math.h>
#include "ftab.h"

// print a function table
ostream& FuncTab::Print(ostream& o, int) const
{
  o << "FuncTab(" << this << ")";
  return o;
}

// print a function table
ostream& operator<<(ostream& os, const FuncTab& ft)
{
  return ft.Print(os);
}

#if 0
// lookup a precomputed value in a function table
// note: there is no interpolation performed!
double FuncTab::operator()(double v) const
{
  assert(v >= 0.0 && v <= _max);
  return _data[(int)(v * _index)];
}
#endif

// return the nth value in a function table
double FuncTab::operator[](int n) const
{
  assert(n >= 0 && n < _size);
  return _data[n];
}

// copy contents of a function table
void FuncTab::Copy(const FuncTab& f)
{
  _size = f._size;
  _index = f._index;
  _gran = f._gran;
  _data = new double[f._size];
  _max = f._max;
  _fn = f._fn;
  MemCopy(f._data, _data, sizeof(*_data)*_size);
}

// create an empty function table
FuncTab::FuncTab()
: _size(0), _index(0), _gran(0), _data(0), _max(0), _fn(0)
{
}

// create a function table by copying another one
FuncTab::FuncTab(const FuncTab& f)
: _size(f._size), _index(f._index), _gran(f._gran),
  _data(new double[f._size]), _max(f._max), _fn(f._fn)
{
  MemCopy(f._data, _data, sizeof(*_data)*_size);
}

// create a function table with the given function, granularity and
// maximum range value. (e.g. function is computed for all values
// between 0 and max (with intervals of gran size) and stored in
// a table)
FuncTab::FuncTab(DoubleFn1 fn, double max, double gran)
: _gran(gran), _max(max), _fn(fn)
{
  _size = (int)(_max/_gran) + 1;
  _index = _size / _max;
  _data = new double[_size];
  for(int i = 0; i < _size; i++)
    _data[i] = fn(i * _gran);
}


// lookup function value at v
double InterpFuncTab::operator()(double v) const
{
  v *= Index();
  int before = (int)floor(v);
  int after = (int)ceil(v);
  double fb = (*this)[before];
  if(before != after)
    return (v - before)*((*this)[after] - fb) + fb;
  else
    return fb;
}
