////////////////////////////////////////////////////////////////////////////
//
// interp.h - linear and hermite cubic interpolation operations
//
////////////////////////////////////////////////////////////////////////////
#ifndef _INTERP_H_
#define _INTERP_H_

#include "real.h"

// linear interpolation
inline Real Lerp(Real dx, Real x1, Real x2)
{
  return (1.0 - dx) * x1 + dx * x2;
}

// cubic Hermite interpolation
inline Real Hermite(Real p0, Real p1, Real r0, Real r1, Real t)
{
  return p0 * ((2.0 * t - 3.0) * t * t + 1.0) + 
         p1 * (-2.0 * t + 3.0) * t * t +
	 r0 * ((t - 2.0) * t + 1.0) * t +
	 r1 * (t - 1.0) * t * t;
}

#endif
