#include <stream.h>
#include "matrix3d.h"
#include "trig.h"

// print a matrix
ostream& operator<<(ostream& os, const Matrix3d& m)
{
  return m.Print(os);
}

// print a matrix
ostream& Matrix3d::Print(ostream& os, int) const
{
  os << "Matrix3d(";
  for(int i = 0; i < 4; i++)
    for(int j = 0; j < 4; j++)
      os << _data[i][j] << (j == 3 ? ", " : " ");
  os << ")";
  return os;
}

// apply a vector to a matrix and return the resultant vector
Vector3d Matrix3d::operator*(const Vector3d& a) const
{
  return Vector3d(a.i * _data[0][0] + a.j * _data[1][0] + a.k * _data[2][0],
		  a.i * _data[0][1] + a.j * _data[1][1] + a.k * _data[2][1],
		  a.i * _data[0][2] + a.j * _data[1][2] + a.k * _data[2][2]);
}

// multiply two matrices the stupid way
Matrix3d Matrix3d::operator*(const Matrix3d& m) const
{
  Matrix3d newm;
  for(int i = 0; i < 4; i++) {
    for(int j = 0; j < 4; j++) {
      Real sum = 0.0;
      for(int k = 0; k < 4; k++)
	sum += _data[i][k]*m._data[k][j];
      newm._data[i][j] = sum;
    }
  }
  return newm;
}

// generate a 3d rotation matrix around the x axis
RotateX3d::RotateX3d(Real ang)
{
  (*this)(0, 0) = 1.0;
  (*this)(1, 1) = fcos(ang);
  (*this)(1, 2) = -fsin(ang);
  (*this)(2, 1) = fsin(ang);
  (*this)(2, 2) = fcos(ang);
  (*this)(3, 3) = 1.0;
}

// generate a 3d rotation matrix around the z axis
RotateZ3d::RotateZ3d(Real ang)
{
  (*this)(0, 0) = fcos(ang);
  (*this)(0, 1) = fsin(ang);
  (*this)(1, 0) = -fsin(ang);
  (*this)(1, 1) = fcos(ang);
  (*this)(2, 2) = 1.0;
  (*this)(3, 3) = 1.0;
}

// generate a 3d rotation matrix around the y axis
RotateY3d::RotateY3d(Real ang)
{
  (*this)(0, 0) = fcos(ang);
  (*this)(0, 2) = -fsin(ang);
  (*this)(2, 0) = fsin(ang);
  (*this)(2, 2) = fcos(ang);
  (*this)(1, 1) = 1.0;
  (*this)(3, 3) = 1.0;
}

// generate a 3d translation matrix
Translate3d::Translate3d(Real dx, Real dy, Real dz)
{
  (*this)(0, 3) = dx;
  (*this)(1, 3) = dy;
  (*this)(2, 3) = dz;
  (*this)(0, 0) = 1.0;
  (*this)(1, 1) = 1.0;
  (*this)(2, 2) = 1.0;
  (*this)(3, 3) = 1.0;
}

// generate a 3d scaling matrix
Scale3d::Scale3d(Real sx, Real sy, Real sz)
{
  (*this)(0, 0) = sx;
  (*this)(1, 1) = sy;
  (*this)(2, 2) = sz;
  (*this)(3, 3) = 1.0;
}
