////////////////////////////////////////////////////////////////////////////
//
// matrix3d.h - 4x4 matrix class used for 3d transformations
//
////////////////////////////////////////////////////////////////////////////
#ifndef _MATRIX3D_H_
#define _MATRIX3D_H_

#include "real.h"
#include "vector.h"

// 3d transformation matrix class
class Matrix3d {
 public:
  Vector3d operator*(const Vector3d&) const;  // apply vector to a matrix
  Matrix3d operator*(const Matrix3d&) const;  // matrix multiply

  // access matrix elements
  Real& operator()(int i, int j) { return _data[i][j]; }

  // create new empty matrix
  Matrix3d() { MemClear(_data, sizeof(_data)); }

  // destroy a matrix
  ~Matrix3d() { }

  // print matrix
  friend ostream& operator<<(ostream&, const Matrix3d&);
  ostream& Print(ostream& o, int=0) const;
 private:
  Real _data[4][4];  // matrix data
};

// 3d translation matrix
class Translate3d : public Matrix3d {
 public:
  Translate3d(Real, Real, Real);
};

// 3d scale matrix
class Scale3d : public Matrix3d {
 public:
  Scale3d(Real, Real, Real);
};

// 3d x-axis rotation matrix
class RotateX3d : public Matrix3d {
 public:
  RotateX3d(Real);
};

// 3d y-axis rotation matrix
class RotateY3d : public Matrix3d {
 public:
  RotateY3d(Real);
};

// 3d z-axis rotation matrix
class RotateZ3d : public Matrix3d {
 public:
  RotateZ3d(Real);
};

#endif
