////////////////////////////////////////////////////////////////////////////
//
// plane.h - simple plane class
//
////////////////////////////////////////////////////////////////////////////
#ifndef _PLANE_H_
#define _PLANE_H_

#include "real.h"
#include "point.h"
#include "vector.h"

// Plane defined by a point and a normal vector
// Note: normal is always in "normalized" form
class Plane {
 public:
  // return the normal vector of this plane
  const Vector3d& Normal() const { return _n; }

  // return point on the plane
  const Point3d& Point() const { return _p; }

  // check if given point is below the plane
  Boolean operator>(const Point3d& p) const
    { return MakeBoolean(_n.Dot(Vector3d(p - _p)) > 0.0); }

  // check if given point is above the plane
  Boolean operator<(const Point3d& p) const
    { return MakeBoolean(_n.Dot(Vector3d(p - _p)) < 0.0); }

  // check if given point is on the plane
  Boolean operator==(const Point3d& p) const
    { return MakeBoolean(_n.Dot(Vector3d(p - _p)) == 0.0); }

  // create xy plane
  Plane() : _n(0.0, 0.0, 1.0) { }

  // create plane from point and a normal
  Plane(const Point3d& p, const Vector3d& n)
    : _p(p), _n(n) { _n.Normalize(); }

  // create plane from 3 non-collinear points
  Plane(const Point3d& a, const Point3d& b, const Point3d& c)
    : _p(c) { _n = Vector3d(a - c).Cross(b - c).Normalize(); }

  // destroy a plane
  ~Plane() { }

  // print plane
  friend ostream& operator<<(ostream&, const Plane&);
  ostream& Print(ostream& o, int=0) const;
 private:
  Point3d _p;   // a point on the plane
  Vector3d _n;  // plane's normal vector
};

#endif
