////////////////////////////////////////////////////////////////////////////
//
// real.h - defines Real type (either float or double) and some common
//          functions operating on Real types (including random number
//          generators)
//
////////////////////////////////////////////////////////////////////////////
#ifndef _REAL_H_
#define _REAL_H_

#include <math.h>
#include <float.h>
#include "misc.h"

// if defined means use precomputed sin/cos values instead of functions
//#define TRIG_TABLE

/* use real everywhere in case we want to change things to double */
typedef float Real;
//typedef double Double;

typedef Real (*RealFn1)(Real);             // real functions of 1 arg
typedef Real (*RealFn2)(Real, Real);       // real functions of 2 args
typedef Real (*RealFn3)(Real, Real, Real); // real functions of 3 args

// max real value
#define RealMax FLT_MAX
//#define RealMax DBL_MAX

/* compare signs of two numbers */
inline Boolean SignEq(Real a, Real b)
{
  return MakeBoolean((a >= 0.0 && b >= 0.0) || (a <= 0.0 && b <= 0.0));
}

// round a real number
inline Real round(Real a)
{
  Real fa = floor(a);
  return (fabs(a - fa) < 0.5) ? fa : ceil(a);
}

// make sure a is in between min and max
inline int fixrng(int a, int min, int max)
{
  if(a < min) a = min;
  if(a >= max) a = max - 1;
  return a;
}

// make sure a is in between min and max
inline Real fixrrng(Real a, Real min, Real max)
{
  if(a < min) a = min;
  if(a > max) a = max;
  return a;
}

// functions to generate random numbers in [0, x)
extern "C" double drand48();
extern "C" long lrand48();

extern double grand();         // gaussian random number [grand_min, grand_max]
extern double ngrand(double x);// normalized gaussian random number [0, x]
extern double grand_min;       // minimum value for grand
extern double grand_max;       // maximum value for grand

// integer random number in [0, x)
inline int irand(int x = 0) { return x ? lrand48() % x : lrand48(); }

// real random number in [0, x)
inline double rrand(double x) { return x*drand48(); }

// initialize random number generators (if global ctors can't be used)
extern int InitRand(int seed = 0);

// compute square and cube of numbers
inline Real sqr(Real x) { return x*x; }
inline Real cub(Real x) { return x*x*x; }

// compute gaussian at given t and sigma
extern Real Gaussian(Real sigma, Real t);

#endif
