#include "color.h"

// compare two colors for equality
Boolean Color::operator==(const Color& c) const
{
  // first make sure they are the same kind of color
  if(c.IsA(ObjectId()))
    return MakeBoolean(_r == c._r && _g == c._g && _b == c._b);
  return False;
}

// multiply RGB color by constant s
RGB RGB::operator*(Real s) const
{
  return RGB(s * r(), s * g(), s * b());
}

// multiply two RGB colors
RGB RGB::operator*(const Color& cc) const
{
  RGB c = (RGB)cc;
  return RGB(c.r() * r(), c.g() * g(), c.b() * b());
}

// add constant factor to RGB color
RGB RGB::operator+(Real s) const
{
  return RGB(s + r(), s + g(), s + b());
}

// add two RGB colors
RGB RGB::operator+(const Color& cc) const
{
  RGB c = (RGB)cc;
  return RGB(c.r() + r(), c.g() + g(), c.b() + b());
}

void RGB::Brighten(Real f)
{
  Real rf = f * 0.299/0.587; // f;
  Real gf = f;
  Real bf = f * 0.114/0.587; // f;

  if((rf *= r()) > 1.0) rf = 1.0;
  if((gf *= g()) > 1.0) gf = 1.0;
  if((bf *= b()) > 1.0) bf = 1.0;

  Set(rf, gf, bf);
}

void RGB::Darken(Real f)
{
  assert(f > 0.0);
  Real rf = 1.0 + ((f - 1.0) * 0.299/0.587); //f;
  Real gf = f;
  Real bf = 1.0 + ((f - 1.0) * 0.114/0.587); //f;
  Set(r()/rf, g()/gf, b()/bf);
}

void RGB::Scale(Real f)
{
  Real rf = f * r();
  Real gf = f * g();
  Real bf = f * b();
  if(rf > 1.0) rf = 1.0;
  if(gf > 1.0) gf = 1.0;
  if(bf > 1.0) bf = 1.0;
  Set(rf, gf, bf);
}


// print an RGB color
ostream& RGB::Print(ostream& os,int) const
{
  Object::Print(os);
  os << "(" << r() << ", " << g() << ", " << b() << ")";
  return os;
}




// multiply gray scale color by a constant factor
GrayScale GrayScale::operator*(Real s) const
{
  return GrayScale(s * Intensity());
}

// multiply two gray scales
GrayScale GrayScale::operator*(const Color& c) const
{
  return GrayScale(c.Intensity() * Intensity());
}

// add constant factor to GrayScale color
GrayScale GrayScale::operator+(Real s) const
{
  return GrayScale(s + Intensity());
}

// add two GrayScale colors
GrayScale GrayScale::operator+(const Color& c) const
{
  return GrayScale(Intensity() + c.Intensity());
}

// print a gray scale
ostream& GrayScale::Print(ostream& os, int) const
{
  Object::Print(os);
  os << "(" << Intensity() << ")";
  return os;
}



// multiply bit color by a constant factor (really an "and" operation)
BitColor BitColor::operator*(Real s) const
{
  return BitColor(s * Intensity());
}

// multiply two bit colors (really an "and" operation)
BitColor BitColor::operator*(const Color& c) const
{
  return BitColor(c.Intensity() * Intensity());
}

// add constant factor to BitColor color
BitColor BitColor::operator+(Real s) const
{
  return BitColor(s + Intensity());
}

// add two BitColor colors
BitColor BitColor::operator+(const Color& c) const
{
  return BitColor(Intensity() + c.Intensity());
}

// print a bit color
ostream& BitColor::Print(ostream& os, int) const
{
  Object::Print(os);
  os << "(" << Bit() << ")";
  return os;
}
