////////////////////////////////////////////////////////////////////////////
//
// light.h - colored light class (color + direction)
//
////////////////////////////////////////////////////////////////////////////
#ifndef _LIGHT_H_
#define _LIGHT_H_

#include <stream.h>
#include "vector.h"
#include "color.h"

// light class
class Light {
 public:
  // convert light to vector by returning it's direction
  operator Vector3d() const { return _dir; }

  // return intensity of light
  Real Intensity() const { return _c.Intensity(); }

  // return color of the light
  RGB& color() const { return ((Light*)this)->_c; }

  // return direction of light
  Vector3d& Direction() const { return ((Light*)this)->_dir; }

  // create new white light given color and direction
  Light(const Vector3d& v) : _c(RGB(1.0, 1.0, 1.0)), _dir(v) { _dir.Normalize(); }

  // create new light given color and direction
  Light(const RGB& c, const Vector3d& v) : _c(c), _dir(v) { _dir.Normalize(); }

  ostream& Print(ostream& os, int=0) const
    {
      os << "Light(" << _c << ", " << _dir << ")";
      return os;
    }
 private:
  RGB _c;         // color of light
  Vector3d _dir;  // direction of light
};

#endif
