////////////////////////////////////////////////////////////////////////////
//
// palette.h - color palette class
//
////////////////////////////////////////////////////////////////////////////
#ifndef _PALETTE_H_
#define _PALETTE_H_

#include "Object.h"
#include "color.h"
#include "real.h"

// color palette (can contain any type of color)
class Palette : public Object {
  DeclareStandardMethods(Palette, Object);
 public:
  int Size() const { return _size; }   // return size of the palette
  int operator[](const Color&) const;  // lookup color index of given color

  // return nth color in palette
  Color& operator[](int i) const { return ((Palette*)this)->_colors[i]; }

  void Scale(Real, int = 0, int = -1);
  void Brighten(Real, int = 0, int = -1);
  void Darken(Real, int = 0, int = -1);

  // destroy palette
  ~Palette() { delete[] _colors; }
 protected:
  // create empty palette
  Palette() : _size(0), _colors(0) { }

  // create palette of given size and optional of color values
  Palette(int size, Color* c) : _size(size), _colors(c) { }

  // return pointer to private color array
  Color* colors() { return _colors; }
 private:
  int _size;       // palette size
  Color* _colors;  // color array
};

// RGB color palette
class RGB_Palette : public Palette {
  DeclareStandardMethodsNoPrint(RGB_Palette, Palette);
 public:
  // return nth color as an RGB color
  RGB& operator[](int i) { return (RGB&)(colors()[i]); }

  // create empty RGB palette
  RGB_Palette() { }

  // create RGB palette of given size with unitialized colors
  RGB_Palette(int size) : Palette(size, new RGB[size]) { }
};

#endif
