#include "terrain.h"

// initialize terrain palette
void TerrainPalette::Init()
{
  int sea_size = (int)(Size() * _sea);
  int beach_size = (int)(Size() * _beach) + sea_size;
  int plain_size = (int)(Size() * _plains) + beach_size;
  int low_mnt_size = (int)(Size() * _hill) + plain_size;
  int high_mnt_size = (int)(Size() * _mnts) + low_mnt_size;
  int i;

  /* black */
  (*this)[0] = RGB(0.0, 0.0, 0.0);

  /* sea colors (blue) */
  for(i = 1; i <= sea_size; i++) {
    Real perc = (Real)i / (Real)sea_size;
    (*this)[i] = RGB(0.0, 0.0, 105.0/256.0 * perc + 150.0/256.0);
  }

  /* yellow */
  for(i = sea_size+1; i <= beach_size; i++) {
    Real perc = (Real)(i - sea_size - 1) / (Real)(beach_size - sea_size);
    (*this)[i] = RGB(55.0/256.0 * perc + 200.0/256.0,
		     40.0/256.0 * perc + 170.0/256.0,
		     0.0);
  }

  /* plain size (green) */
  for(i = beach_size+1; i <= plain_size; i++) {
    Real perc = (Real)(i - beach_size - 1)/ (Real)(plain_size - beach_size);
    (*this)[i] = RGB(0.0, 135.0/256.0 * perc + 120.0/256.0, 0.0);
  }

#if 0
  /* low_mnt size (brown) */
  for(i = plain_size+1; i <= low_mnt_size; i++) {
    Real perc = (Real)(i - plain_size - 1) / (Real)(low_mnt_size - plain_size);
    (*this)[i] = RGB(100.0/256.0 * perc + 130.0/256.0,
		     70.0/256.0 * perc + 80.0/256.0,
		     0.0);
  }
#else
  /* low_mnt size (brown) */
  for(i = plain_size+1; i <= low_mnt_size; i++) {
    Real perc = (Real)(i - plain_size - 1) / (Real)(low_mnt_size - plain_size);
    (*this)[i] = RGB(80.0/256.0 * perc + 120.0/256.0,
		     50.0/256.0 * perc + 70.0/256.0,
		     0.0);
  }
#endif

  /* hight_mnt_size size (gray -> white) */
  /*printf("%d/%d\n", high_mnt_size, _size);*/
  for(i = low_mnt_size+1; i < Size(); i++) {
    Real perc = (Real)(i - low_mnt_size - 1) / (Real)(Size() - low_mnt_size);
    (*this)[i] = RGB(0.5 * perc + 0.5, 0.5 * perc + 0.5, 0.5 * perc + 0.5);
  }
}

TerrainPalette::TerrainPalette()
: RGB_Palette(256), _sea(0.5), _beach(0.05),
  _plains(0.3), _hill(0.1), _mnts(0.05)
{
  Init();
}

TerrainPalette::TerrainPalette(Real sea, Real beach, Real plains,
			       Real hill, Real mnts)
: RGB_Palette(256), _sea(sea), _beach(beach),
  _plains(plains), _hill(hill), _mnts(mnts)
{
  // make sure sum of all coefficients is 1.0
  Real* tmp[5];
  Real sum = 0.0, adj = 0.0;
  tmp[0] = &_sea;
  tmp[1] = &_beach;
  tmp[2] = &_plains;
  tmp[3] = &_hill;
  tmp[4] = &_mnts;

  for(int i = 0; i < 5; i++)
    sum += *tmp[i];

  if(sum != 1.0) {
    adj = 1.0 / sum;
    for(int i = 0; i < 5; i++)
      *tmp[i] *= adj;
  }
  Init();
}
