////////////////////////////////////////////////////////////////////////////
//
// terrain.h - defines a simple terrain palette that can be adjusted
//             by different height values for terrain features.
//             Each feature value specifies what percentage of the
//             palette is used for that type of feature. (e.g. if the
//             sea level is 0.6 then 60% of the palette will be used
//             for blues).  The sum of all the feature values should
//             be 1.0.  If it isn't the rest of the values are scaled
//             so that they always sum to 1.0 (Note: this may produce
//             weird results.
//
//             The different features are (in order of appearance in
//             the palette): sea, beach, plains, low mountains and
//             high (e.g. snow capped) mountains.
//
////////////////////////////////////////////////////////////////////////////
#ifndef _TERRAIN_H_
#define _TERRAIN_H_

#include "palette.h"

// terrain palette class
class TerrainPalette : public RGB_Palette {
  DeclareStandardMethodsNoPrint(TerrainPalette, RGB_Palette);
 public:
  // create terrain palette with default feature values
  TerrainPalette();

  // create terrain palette with one or more user specified
  // feature values.
  TerrainPalette(Real sea = 0.5, Real beach = 0.05, Real plain = 0.3,
		 Real hill = 0.1, Real mnts = 0.05);
  ~TerrainPalette() { }
 protected:
  void Init();  // initialize palette
 private:
  Real _sea;      // sea level
  Real _beach;    // beach level
  Real _plains;   // plains level
  Real _hill;     // low mountains level
  Real _mnts;     // high mountains level
};

#endif
